from typing import Any, Union

import h5py

from .base import File
from .errors import Hdf5FileError
from ..file import FilePath


class Hdf5File(File):
    """
    This is a class for managing the reading and writing of hdf5 objects.
    """
    SUPPORTED_FORMATS = ["h5", "hdf5", "hdf"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the Hdf5File class.

        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: Data from the hdf5 file
        """
        h5 = h5py.File(self.path, mode='r')
        return h5

    def write(self, data: Any, **kwargs) -> Exception:
        """
        Writes data to file (WARNING: not supported, raises exception).

        :param data: (python object) data to be written to file
        :return: None
        """
        raise Hdf5FileError('write method not yet implemented')



from typing import Any, Union

from .base import File
from .errors import KerasModelFileError
from ..path import FilePath


class KerasModelFile(File):
    """
    This is a class for managing the reading and writing of tensorflow.keras.Model objects.
    """
    SUPPORTED_FORMATS = ["keras_model"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the KerasModelFile class.
        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, custom_objects: dict = {}, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: Data from the json file
        """
        try:
            from tensorflow.keras.models import load_model
        except ImportError:
            raise KerasModelFileError(
                message="loading a keras model relies on tensorflow, to install the correct version run the command: 'file-install-tensorflow'"
            )
        return load_model(self.path, compile=False, custom_objects=custom_objects)

    def write(self, data: Any, **kwargs) -> None:
        """
        Writes data to file.
        :param data: (python dict) data to be written to file
        """
        try:
            from tensorflow.keras.models import save_model
        except ImportError:
            raise KerasModelFileError(
                message="saving a keras model relies on tensorflow, to install the correct version run the command: 'file-install-tensorflow'"
            )
        save_model(data, self.path, include_optimizer=False, save_format='h5')



from typing import Dict, Union

import yaml

from .base import File
from .errors import YamlFileError
from ..path import FilePath


class YmlFile(File):
    """
    This is a class for managing the reading and writing of yaml files.
    """
    SUPPORTED_FORMATS = ["yml"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the YmlFile class.

        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Dict:
        """
        Gets data from file defined by the file path.

        :return: Data from the yaml file
        """
        yml_path = str.encode(self.path)
        raw_data = open(yml_path, 'rb')
        loaded_data = yaml.load(raw_data, Loader=yaml.FullLoader)
        raw_data.close()
        return loaded_data

    def write(self, data: Dict, **kwargs) -> None:
        """
        Writes data to file.

        :param data: (dict) data to be written to file
        :return: None
        """
        if not isinstance(data, dict):
            raise YamlFileError(message="{} data supplied instead of dict".format(type(data)))
        yml_path = str.encode(self.path)
        file = open(yml_path, 'w')
        yaml.dump(data, file)
        file.close()



import re
from typing import Union

def monolith_globre_match(pattern: str, string: str) -> Union[re.Match, None]:
    """
    Replicate the behaviour of globre.match.
    """
    return re.match(_glob_to_re(pattern), string)

def _glob_to_re(pat: str) -> str:
    """
    Translate a shell PATTERN to a regular expression.
    SOURCE: https://stackoverflow.com/questions/27726545/python-glob-but-against-a-list-of-strings-rather-than-the-filesystem/72400344#72400344

    Derived from `fnmatch.translate()` of Python version 3.8.13
    SOURCE: https://github.com/python/cpython/blob/v3.8.13/Lib/fnmatch.py#L74-L128
    """

    i, n = 0, len(pat)
    res = ''
    while i < n:
        c = pat[i]
        i = i+1
        if c == '*':
            # -------- CHANGE START --------
            # prevent '*' matching directory boundaries, but allow '**' to match them
            j = i
            if j < n and pat[j] == '*':
                res = res + '.*'
                i = j+1
            else:
                res = res + '[^/]*'
            # -------- CHANGE END ----------
        elif c == '?':
            # -------- CHANGE START --------
            # prevent '?' matching directory boundaries
            res = res + '[^/]'
            # -------- CHANGE END ----------
        elif c == '[':
            j = i
            if j < n and pat[j] == '!':
                j = j+1
            if j < n and pat[j] == ']':
                j = j+1
            while j < n and pat[j] != ']':
                j = j+1
            if j >= n:
                res = res + '\\['
            else:
                stuff = pat[i:j]
                if '--' not in stuff:
                    stuff = stuff.replace('\\', r'\\')
                else:
                    chunks = []
                    k = i+2 if pat[i] == '!' else i+1
                    while True:
                        k = pat.find('-', k, j)
                        if k < 0:
                            break
                        chunks.append(pat[i:k])
                        i = k+1
                        k = k+3
                    chunks.append(pat[i:j])
                    # Escape backslashes and hyphens for set difference (--).
                    # Hyphens that create ranges shouldn't be escaped.
                    stuff = '-'.join(s.replace('\\', r'\\').replace('-', r'\-')
                                     for s in chunks)
                # Escape set operations (&&, ~~ and ||).
                stuff = re.sub(r'([&~|])', r'\\\1', stuff)
                i = j+1
                if stuff[0] == '!':
                    # -------- CHANGE START --------
                    # ensure sequence negations don't match directory boundaries
                    stuff = '^/' + stuff[1:]
                    # -------- CHANGE END ----------
                elif stuff[0] in ('^', '['):
                    stuff = '\\' + stuff
                res = '%s[%s]' % (res, stuff)
        else:
            res = res + re.escape(c)
    return r'(?s:%s)\Z' % res



from unittest import TestCase, main
from unittest.mock import patch

from monolith_filemanager.file.yml_file import YmlFile, YamlFileError


class TestYmlFile(TestCase):

    @patch("monolith_filemanager.file.yml_file.File.__init__")
    def test___init__(self, mock_file):
        YmlFile(path="test")
        mock_file.assert_called_once_with(path="test")

    @patch("monolith_filemanager.file.yml_file.open")
    @patch("monolith_filemanager.file.yml_file.yaml")
    @patch("monolith_filemanager.file.yml_file.File.__init__")
    def test_read(self, mock_file, mock_yaml, mock_open):
        mock_file.return_value = None
        test = YmlFile(path="test")
        test.path = "test"
        out_come = test.read()

        mock_open.assert_called_once_with(str.encode("test"), 'rb')
        mock_open.return_value.close.assert_called_once_with()
        mock_yaml.load.assert_called_once_with(mock_open.return_value, Loader=mock_yaml.FullLoader)
        self.assertEqual(mock_yaml.load.return_value, out_come)

    @patch("monolith_filemanager.file.yml_file.open")
    @patch("monolith_filemanager.file.yml_file.yaml")
    @patch("monolith_filemanager.file.yml_file.File.__init__")
    def test_write(self, mock_file, mock_yaml, mock_open):
        mock_file.return_value = None
        test = YmlFile(path="test")
        data = {"one": 1, "two": 2}
        test.path = "test"
        test.write(data=data)

        mock_open.assert_called_once_with(str.encode("test"), 'w')
        mock_yaml.dump.assert_called_once_with(data, mock_open.return_value)
        mock_open.return_value.close.assert_called_once_with()

        with self.assertRaises(YamlFileError) as context:
            test.write(data="test")

        self.assertEqual("<class 'str'> data supplied instead of dict", str(context.exception))
        mock_open.assert_called_once_with(str.encode("test"), 'w')
        mock_yaml.dump.assert_called_once_with(data, mock_open.return_value)
        mock_open.return_value.close.assert_called_once_with()


if __name__ == "__main__":
    main()



from abc import ABC, abstractmethod
from typing import Union

from .errors import BaseFileError
from ..path import FilePath


class File(ABC):
    """
    This is a class for managing the base attributes of a file class.

    Attributes:
        path (FilePath): path to file
    """
    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the File base path.

        :param path: (str/FilePath) path to file
        """
        if isinstance(path, FilePath):
            self.path: FilePath = path
        elif isinstance(path, str):
            self.path: FilePath = FilePath(path)
        else:
            raise BaseFileError("{} cannot be configured into a path, please pass FilePath object or string".format(
                path))

    @abstractmethod
    def read(self, **kwargs) -> None:
        pass

    @abstractmethod
    def write(self, data, **kwargs) -> None:
        pass

    @staticmethod
    def supports_s3() -> bool:
        return False



import os
from abc import ABC, abstractmethod
from typing import Any, Tuple, List, Union, Optional

from monolith_filemanager.file import FileMap
from monolith_filemanager.path import FilePath


class Base(ABC):
    """
    This is a class for managing the base functions for other adapters.

    Attributes:
        path (FilePath): the full path to the file (FilePath is defined in the file_manager_factory)
        file_types (FileMap: dict): dictionary of the different file types with read and write functionality
        config (Optional[str]): configuration for the adapter
    """
    def __init__(self, file_path: Union[FilePath]) -> None:
        """
        The constructor for the Base class.

        :param file_path: (FilePath) the full path to the file (FilePath is defined in the file_manager_factory)
        """
        self.path: FilePath = file_path
        self.file_types: FileMap = FileMap()
        self._s3: bool = False
        self.config: Optional[str] = None

    def define_config(self, config: str) -> None:
        """
        Defines the config attribute.

        :param config: (str) the configuration dictionary that is attached to the self.config attribute
        :return: None
        """
        self.config = config

    @abstractmethod
    def read_file(self, **kwargs) -> Any:
        """
        Placeholder for reading from file

        :return: data from file
        """
        pass

    @abstractmethod
    def read_raw_file(self) -> bytes:
        """
        Placeholder for reading raw data from file

        :return: (bytes) data from file
        """
        pass

    @abstractmethod
    def custom_read_file(self, custom_read_function) -> Any:
        pass

    @abstractmethod
    def write_file(self, data, **kwargs) -> None:
        """
        Placeholder for writing data to a file.

        :param data: data that has to be written to a file
        :return: None
        """
        pass

    @abstractmethod
    def write_raw_file(self, data: bytes) -> None:
        """
        Placeholder for writing raw data to a file.

        :param data: data that has to be written to a file
        :return: None
        """
        pass

    @abstractmethod
    def delete_file(self, path=None) -> None:
        pass

    @abstractmethod
    def delete_folder(self, path=None) -> None:
        pass

    @abstractmethod
    def write_stream(self, stream) -> None:
        pass

    @abstractmethod
    def create_directory_if_not_exists(self) -> None:
        pass

    @abstractmethod
    def exists(self) -> bool:
        pass

    @abstractmethod
    def ls(self) -> Tuple[List[str], List[str]]:
        """
        Placeholder for returning list of files and subdirectories.

        :return: Tuple of list of subdirectories and list of files within folder
        """
        pass

    @abstractmethod
    def copy_folder(self, new_folder: str) -> None:
        pass

    def batch_delete(self, paths: List[str]) -> None:
        """
        Batch delete files or folders within the parent directory of self.path.

        :param paths: (List[str]) file/folder paths within self.path to be deleted
        :return: None
        """
        for path in paths:
            del_path = FilePath(f"{self.path}/{path}")
            # Assume that any path with no extension represents a folder. We must do this in order
            # to append a trailing slash to the path later to prevent spurious folder deletion as
            # a result of the use of the Prefix filter in boto3. Without this logic, deleting folder
            # DirA would also delete DirA* (DirAB, DirABC etc.).
            _, ext = os.path.splitext(del_path)

            if ext:
                self.delete_file(path=del_path)

            self.delete_folder(path=del_path)

    @abstractmethod
    def rename_file(self, new_name: str) -> None:
        pass

    @abstractmethod
    def rename_folder(self, new_name: str) -> None:
        pass

    @abstractmethod
    def move_file(self, destination_folder: str) -> None:
        pass

    @abstractmethod
    def move_folder(self, destination_folder: str) -> None:
        pass



import tempfile
from unittest import TestCase, main
from unittest.mock import patch, MagicMock

import dask.dataframe as dd
import pandas as pd
from numpy.testing import assert_array_equal
from pandas.testing import assert_frame_equal, assert_index_equal
from parameterized import parameterized

from monolith_filemanager.file.errors import PandasFileError
from monolith_filemanager.file.pandas_file import PandasFile

file_types = PandasFile.SUPPORTED_FORMATS
CHUNK_SIZE = '256MB'
STORAGE_OPTIONS = {'config_kwargs': {'max_pool_connections': 32}, 'skip_instance_cache': True}


class TestPandasFile(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.test_dir = tempfile.TemporaryDirectory()
        cls.example_df = pd.DataFrame({'a': [1, 2], 'b': [3, 4], 'c': [5, 6]})
        cls.example_df.to_parquet(f'{cls.test_dir.name}/data.parquet')
        cls.example_df.to_csv(f'{cls.test_dir.name}/data.csv', index=False)
        cls.example_df.to_excel(f'{cls.test_dir.name}/data.xls', index=False)
        cls.example_df.to_excel(f'{cls.test_dir.name}/data.xlsx', index=False)
        cls.example_df.to_csv(f'{cls.test_dir.name}/data.dat', index=False, sep=',')
        cls.example_df.to_csv(f'{cls.test_dir.name}/data.data', index=False, sep=',')

        # Example dask df with index of [0, 0, 1, 1]
        cls.example_dask_df = \
            dd.from_pandas(
                pd.concat([cls.example_df, cls.example_df], axis=0),
                npartitions=1,
                sort=False
            )
        cls.example_dask_df.to_parquet(f'{cls.test_dir.name}/dask-data.parquet')

    @classmethod
    def tearDownClass(cls):
        cls.test_dir.cleanup()

    @patch("monolith_filemanager.file.pandas_file.File.__init__")
    def test___init__(self, mock_file_init):
        PandasFile(path="test")
        mock_file_init.return_value = None
        mock_file_init.assert_called_once_with(path="test")

    @patch("monolith_filemanager.file.pandas_file.PandasFile.__init__")
    def test_write(self, mock_init):
        mock_init.return_value = None
        test = PandasFile(path="test")

        test.path = MagicMock()
        test_data = pd.DataFrame([{"one": 1, "two": 2}, {"one": 1, "two": 2}])
        test_data_dask = dd.from_pandas(test_data, npartitions=1)

        with self.assertRaisesRegex(ValueError, 'Failed writing dataframe to file\. (.*)'):
            test.write(data=test_data, repartition=False, scheduler='test_scheduler')

        with self.assertRaises(Exception):
            test.write(data="test")
        with self.assertRaises(Exception):
            test.write(data=1)
        with self.assertRaises(Exception):
            test.write(data=[])

    @patch("monolith_filemanager.file.pandas_file.PandasFile.__init__", return_value=None)
    def test_write_with_callback(self, _):
        test = PandasFile(path="test")
        cb = MagicMock()

        test.path = MagicMock()
        test_data = pd.DataFrame([{"one": 1, "two": 2}, {"one": 1, "two": 2}])
        test_data_dask = dd.from_pandas(test_data, npartitions=1)

        with self.assertRaisesRegex(ValueError, 'Failed writing dataframe to file\. (.*)'):
            test.write(data=test_data, cb=cb)

        with self.assertRaises(Exception):
            test.write(data="test")
        with self.assertRaises(Exception):
            test.write(data=1)
        with self.assertRaises(Exception):
            test.write(data=[])

        cb.__enter__.assert_called_once()
        cb.__exit__.assert_called_once()

    @patch("monolith_filemanager.file.pandas_file.PandasFile._read_dask", return_value=None)
    @patch("monolith_filemanager.file.pandas_file.PandasFile.__init__", return_value=None)
    def test_read_lazy(self, _, mock__read_dask):
        PandasFile(path='test').read(lazy=True)
        mock__read_dask.assert_called_once_with(CHUNK_SIZE, storage_options=STORAGE_OPTIONS)

    @patch("monolith_filemanager.file.pandas_file.PandasFile._read_dask", return_value=MagicMock())
    @patch("monolith_filemanager.file.pandas_file.PandasFile.__init__", return_value=None)
    def test_read_eager(self, _, mock__read_dask):
        PandasFile(path='test').read(lazy=False)
        mock__read_dask.assert_called_once_with(CHUNK_SIZE, storage_options=STORAGE_OPTIONS)

    @patch("monolith_filemanager.file.pandas_file.PandasFile.__init__", return_value=None)
    def test__read_dask_invalid_filetype(self, _):
        test = PandasFile(path='test')
        test.path = MagicMock()
        test.path.file_type = 'INVALID FILETYPE'

        with self.assertRaises(PandasFileError):
            _ = test._read_dask(chunk_size=1)

    @parameterized.expand([(f, lazy) for f in file_types for lazy in (True, False)])
    def test_read_functional(self, file_type, lazy):
        file = PandasFile(path=f'{self.test_dir.name}/data.{file_type}')

        if file_type in ('dat', 'data'):
            df = file.read(lazy=lazy, sep=',')
        else:
            df = file.read(lazy=lazy)

        if lazy:
            self.assertIsInstance(df, dd.DataFrame)
            df = df.compute()

        self.assertIsInstance(df, pd.DataFrame)
        assert_frame_equal(df, self.example_df)

    def test_read_dask_saved_file_eager_reset_index(self):
        """
        Dask saves partitioned dataframe folders with a different index to how pandas might store them.
        This test checks that the index returned with file.read(lazy=False) has a consistent linear index.
        """
        file = PandasFile(path=f'{self.test_dir.name}/dask-data.parquet')
        df = file.read(reset_index_if_eager=True)

        self.assertIsInstance(df, pd.DataFrame)
        assert_array_equal(df.values, self.example_dask_df.compute().values)
        assert_index_equal(df.index, pd.Index([0, 1, 2, 3]))

    def test_read_dask_saved_file_eager(self):
        """
        Dask saves partitioned dataframe folders with a different index to how pandas might store them.
        This test checks that the index returned with file.read(lazy=False) has a consistent linear index.
        """
        file = PandasFile(path=f'{self.test_dir.name}/dask-data.parquet')
        df = file.read(reset_index_if_eager=False)

        self.assertIsInstance(df, pd.DataFrame)
        assert_array_equal(df.values, self.example_dask_df.compute().values)
        assert_index_equal(df.index, pd.Index([0, 1, 0, 1]))

    def test_read_dask_saved_file_lazy_reset_index(self):
        file = PandasFile(path=f'{self.test_dir.name}/dask-data.parquet')
        df = file.read(lazy=True, reset_index_if_eager=True)

        self.assertIsInstance(df, dd.DataFrame)
        assert_frame_equal(df.compute(), self.example_dask_df.compute())

if __name__ == "__main__":
    main()



import subprocess
import sys


def install_boto():
    """
    Installs the boto requirement for the package.

    Returns: None
    """
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'boto3>=1.16.43'])



import os
from typing import Tuple, List, Union, Any

from .bucket_manager import BucketManager
from .errors import V1EngineError
from .file_manager import FileManager
from ..path import FilePath


class V1Engine(BucketManager, FileManager):
    """
    This is the main interface for managing operations for s3 buckets and files.

    Attributes:
        resource (object): object managing the resource for s3 connection
        client (object): object managing the client for s3 connection
    """
    _singleton = None

    def __new__(cls, *args, **kwargs):
        if not cls._singleton:
            cls._singleton = super(V1Engine, cls).__new__(cls, *args, **kwargs)
        return cls._singleton

    def __init__(self) -> None:
        """
        The constructor for the V1Engine class.
        """
        try:
            import boto3
        except ImportError:
            raise V1EngineError(
                message="AWS s3 file management requires boto3 to work. Run the command: 'file-install-aws' to install "
                        "the required modules"
        )
        BucketManager.__init__(self)
        FileManager.__init__(self)
        self.client = boto3.client('s3')
        self.resource = boto3.resource('s3')

    def upload_data(self, storage_path: Union[FilePath, str], data: Any) -> None:
        """
        Uploads serialised data to s3 bucket.

        :param storage_path: (str) storage path from the Notebook.get_path_for_artifact function
        :param data: (bytes) data to be upload
        :return: None
        """
        bucket, file_name, _ = V1Engine._split_s3_path(storage_path)
        self.upload_serialised_data(bucket_name=bucket, file_name=file_name, data=data)

    def upload_data_from_file(self, storage_path: str, file_path: str) -> None:
        """
        Uploads serialised data to s3 bucket.

        :param storage_path: (str) storage path in s3
        :param file_path: (str) file path of file to upload
        :return: None
        """
        bucket, file_name, _ = V1Engine._split_s3_path(storage_path)
        self.upload_file_from_disk(bucket_name=bucket, file_name=file_name, file_path=file_path)

    def create_folder(self, storage_path: str) -> str:
        """
        Creates a "folder" object in the s3 bucket when passed a filepath. Does not upload file.

        :param storage_path: (str) storage path in s3
        :return: (str) new folder name
        """
        bucket, file_name, _ = V1Engine._split_s3_path(storage_path)
        folder_name = "/".join(file_name.split("/")[:-1]) + "/"
        parent_dir = "/".join(file_name.split("/")[:-2])
        files, dirs = self.ls_folder(bucket_name=bucket, file_name=parent_dir)
        dirname = file_name
        folder = dirname.split("/")[-2]
        file_names = [x for x in files.keys()]
        if folder in file_names:
            raise V1EngineError("New folder name already taken by file in this folder")
        if self.file_exists(bucket_name=bucket, file_name=folder_name):
            folder_name = self.increment_folder_name(bucket_name=bucket, dirname=folder_name)
        self.upload_serialised_data(bucket_name=bucket, file_name=folder_name)
        return folder_name

    def increment_folder_name(self, bucket_name: str, dirname: str) -> str:
        """
        Increment folder name suffix if name prefix already exists within parent file object collection.
        :param bucket_name: (str) name of bucket containing the folder object
        :param dirname: (str) folder object name to increment
        :return: (str) new folder name
        """
        count = 2
        prefix = "/".join(dirname.split("/")[:-1])
        new_folder = prefix + f" {count}/"
        while self.file_exists(bucket_name=bucket_name, file_name=new_folder):
            count += 1
            new_folder = prefix + f" {count}/"
        self.client.put_object(Bucket=bucket_name, Key=new_folder)
        return new_folder

    def download_raw_data_file(self, storage_path: str) -> Any:
        """
        Downloads raw data from s3 bucket.

        :param storage_path: (str) storage path in the s3 storage space
        :return: (Any) file in memory (usually serialised)
        """
        bucket, file_name, short_file_name = V1Engine._split_s3_path(storage_path)
        return self.download_file_to_memory(bucket_name=bucket, file_name=file_name)

    def download_data_file(self, storage_path: Union[FilePath, str], file_path: Union[FilePath, str]) -> str:
        """
        Download data file to disk.

        :param storage_path: (Union[FilePath, str]) storage path in the s3 storage space
        :param file_path: (Union[FilePath, str]) path where file is to be saved
        :return: (str) path to where the file is downloaded to
        """
        bucket, file_name, short_file_name = V1Engine._split_s3_path(storage_path)
        output_path = os.path.join(file_path, short_file_name)
        self.download_file_to_disk(bucket_name=bucket, file_name=file_name, file_path=output_path)
        return output_path

    def delete(self, storage_path: Union[FilePath, str], folder=False) -> None:
        """
        Deletes the file from s3 or folder and file contents from s3.

        :param storage_path: (str) storage path in the s3 storage space
        :param folder: (bool)
        :return: None
        """
        bucket, file_name, short_file_name = V1Engine._split_s3_path(storage_path)
        if not folder:
            self.delete_file(bucket_name=bucket, file_name=file_name)
        else:
            self.delete_folder(bucket_name=bucket, file_name=file_name)

    def exists(self, storage_path: Union[FilePath, str]) -> bool:
        """
        Checks to see if the path exists.

        :param storage_path: (Union[FilePath, str]) path to be checked
        :return: (bool) True if path exists, False if not
        """
        bucket, file_name, short_file_name = V1Engine._split_s3_path(storage_path)
        if self.file_exists(bucket_name=bucket, file_name=file_name):
            return True
        if self.folder_exists(bucket_name=bucket, file_name=file_name):
            return True
        return False

    def ls(self, storage_path: Union[FilePath, str]) -> Tuple[dict, List[str]]:
        """
        Lists all the sub directories and sub files belonging to the self.path.

        :param storage_path: (Union[FilePath, str]) path to be inspected
        :return: (Tuple[dict, List[str]]) sub directories and sub files
        """
        bucket, file_name, short_file_name = V1Engine._split_s3_path(storage_path)
        return self.ls_folder(bucket_name=bucket, file_name=file_name)

    @staticmethod
    def _split_s3_path(storage_path: Union[FilePath, str]) -> Tuple[str, str, str]:
        """
        Splits the path into bucket, file name, and short_file_name.

        :param storage_path: (Union[FilePath, str]) path to be split
        :return: (Tuple[str, str, str]) bucket_name, file_name, short_file_name
        """
        path = storage_path.replace("s3://", "")
        path = path.split("/")
        bucket_name = path[0]
        file_name = "/".join(path[1:])
        short_file_name = path[-1]
        return bucket_name, file_name, short_file_name





class FilePathError(Exception):

    def __init__(self, message: str) -> None:
        """
        The constructor for the FilePathError class.

        :param message: (str) the message in which the error is raised under
        """
        super().__init__(message)


class FileManagerError(Exception):

    def __init__(self, message: str) -> None:
        """
        The constructor for the FileManagerError class.

        :param message: (str) the message in which the error is raised under
        """
        super().__init__(message)


class PathMapError(Exception):

    def __init__(self, message: str) -> None:
        """
        The constructor for the PathMapError class.

        :param message: (str) the message in which the error is raised under
        """
        super().__init__(message)



from typing import Union, Optional, Any

from .path import FilePath
from .adapters.base import Base
from .adapters.s3_processes import S3ProcessesAdapter
from .adapters.local_file_processes import LocalFileProcessesAdapter
from .components.path_map import PathMap

def file_manager(file_path: Union[str, FilePath], caching: Optional[Any] = None) -> Base:
    """
    Gets the correct adapter based on the path.

    :param file_path: (str) file path
    :param caching: (Optional[Any]) the CacheManager object to be used
    :return: initialized adapter object
    """
    defined_file_path = FilePath(file_path)

    if defined_file_path.s3 is True:
        return S3ProcessesAdapter(file_path=defined_file_path, caching=caching)

    return LocalFileProcessesAdapter(file_path=defined_file_path)



from botocore.exceptions import ClientError
import os
from typing import Any, Tuple, List, Optional
from urllib.parse import unquote

from .errors import FileManagerError


class FileManager:
    """
    This is a class managing the operations for s3 files.

    Attributes:
        resource (object): object managing the resource for s3 connection (None unless overridden by subclass like V1Engine)
        client (object): object managing the client for s3 connection (None unless overridden by subclass like V1Engine)
    """

    BASE_DIR = os.getcwd()
    FILE_PATH = "/tests/monolith_filemanager.connection_type.s3storage/"

    def __init__(self):
        """
        The constructor for the BucketManager class.
        """
        self.resource = None
        self.client = None

    @staticmethod
    def _prep_remote_name(name: str) -> str:
        """
        Prepares the file name in remote format.

        :param name: (str) name of the file
        :return: (str) name formatted for remote conventions
        """
        if " " in name:
            raise FileManagerError(message="space detected in the file name {}".format(name))
        components = name.replace("./", "").split(".")
        if len(components) != 2:
            raise FileManagerError(message="one and only one full stop should be in the name")
        return components[0] + "-remote." + components[1]

    def upload_file_from_disk(self, bucket_name: str, file_name: str, file_path: str) -> None:
        """
        Uploads file from disk to s3 storage.

        :param bucket_name: (str) name of bucket for file to be stored in
        :param file_name: (str) file name to save file as in s3
        :param file_path: (str) path to file being uploaded
        :return:
        """
        self.client.upload_file(file_path,
                                bucket_name,
                                file_name)

    def upload_serialised_data(self, bucket_name: str, file_name: str, data: Optional[bytes] = None) -> None:
        """
        Uploads serialised data to bucket.

        :param bucket_name: (str) name of bucket for data to be stored in
        :param file_name: (str) name for which the data is stored under
        :param data: (Optional[bytes]) data to be stored in s3
        :return: None
        """
        if data:
            self.client.put_object(Body=data, Bucket=bucket_name, Key=file_name)
        else:
            self.client.put_object(Bucket=bucket_name, Key=file_name)

    def download_file_to_disk(self, bucket_name: str, file_name: str, file_path: str) -> None:
        """
        Downloads file from bucket to disk (alter self.BASE_DIR and self.FILE_PATH to alter download destination).

        :param bucket_name: (str) name of bucket for file to be downloaded from
        :param file_name: (str) name of file being downloaded
        :param file_path: (str) path of where file is stored
        :return: None
        """
        self.client.download_file(bucket_name,
                                  file_name,
                                  file_path)

    def download_file_to_memory(self, bucket_name: str, file_name: str) -> Any:
        """
        Downloads file to memory.

        :param bucket_name: (str) name of bucket for file to be downloaded from
        :param file_name: (str) name of file being downloaded
        :return: (Any) downloaded file
        """
        outcome = self.client.get_object(Bucket=bucket_name, Key=file_name)
        return outcome["Body"].read()

    def delete_file(self, bucket_name: str, file_name: str) -> None:
        """
        Deletes file from bucket.

        :param bucket_name: (str) name of bucket for file
        :param file_name: (str) name of file being deleted
        :return: None
        """
        bucket = self.resource.Bucket(bucket_name)
        bucket.objects.filter(Prefix=file_name).delete()

    def delete_folder(self, bucket_name: str, file_name: str) -> None:
        """
        Deletes file from bucket.

        :param bucket_name: (str) name of bucket for file
        :param file_name: (str) name of file being deleted
        :return: None
        """
        self.delete_file(bucket_name=bucket_name, file_name=f"{file_name}/")

    def file_exists(self, bucket_name, file_name) -> bool:
        """
        Check if file exists in a bucket.

        :param bucket_name: (str) name of bucket for file
        :param file_name: (str) name of file to search for
        :return: None
        """
        try:
            self.client.head_object(Bucket=bucket_name, Key=file_name)
            return True
        except ClientError:
            return False
        
    def folder_exists(self, bucket_name, file_name) -> bool:
        try:
            if 'Contents' in self.client.list_objects(Bucket=bucket_name, Prefix=file_name, MaxKeys=1):
                return True
            return False
        except ClientError:
            return False

    def ls_folder(self, bucket_name: str, file_name: str) -> Tuple[dict, List[str]]:
        """
        Lists all the sub directories and sub files belonging to the self.path.

        :param bucket_name: (str) the name of the bucket
        :param file_name: (str) the name of the file
        :return: (Tuple[List[str], List[str]]) sub files with metadata and sub directories
        """
        paginator = self.client.get_paginator('list_objects')
        prefix = file_name
        if prefix and not prefix.endswith("/"):
            prefix += "/"
        pages = paginator.paginate(Bucket=bucket_name, Prefix=prefix, Delimiter='/')
        dirs = []
        files = {}

        for result in pages:
            dirs += [unquote(x.get('Prefix').split("/")[-2]) for x in result.get('CommonPrefixes', []) if x is not None]
            if result.get("Contents"):
                for x in result.get("Contents"):
                    last_modified, size = self.get_file_info(x)
                    files[unquote(x.get('Key').split("/")[-1])] = {"last_modified": last_modified, "size": size}
        return files, dirs

    @staticmethod
    def get_file_info(s3_object: dict) -> Tuple[str, int]:
        """
        Returns last modified date and file size in bytes.

        :param s3_object: (dict) the s3 file object
        :return: (Tuple[str, int]) last_modified datetime and file size in bytes
        """
        last_mod = s3_object.get('LastModified').strftime("%Y-%m-%d %H:%M:%S")
        size = s3_object.get('Size')
        return last_mod, size



from unittest import TestCase, main
from unittest.mock import patch, MagicMock, call
from monolith_filemanager.file import FileMap, Singleton


class TestFileMap(TestCase):

    def setUp(self):
        Singleton._instances = {}

    def tearDown(self):
        Singleton._instances = {}

    @patch("monolith_filemanager.file.FileMap.init_bindings")
    def test___init__(self, mock_init_bindings):
        test = FileMap()
        test_two = FileMap()

        self.assertEqual(id(test), id(test_two))

        test["one"] = 1
        self.assertEqual(test, test_two)
        mock_init_bindings.assert_called_once_with()

        test = FileMap()
        Singleton._instances = {}
        test_two = FileMap()
        self.assertNotEqual(id(test), id(test_two))
        Singleton._instances = {}
        mock_init_bindings.assert_has_calls = [call(), call()]

    @patch("monolith_filemanager.file.FileMap.__init__")
    def test_add_binding(self, mock_init):
        mock_init.return_value = None
        test = FileMap()
        mock_file_object = MagicMock()
        mock_file_object.SUPPORTED_FORMATS = ["csv", "txt"]
        test.add_binding(file_object=mock_file_object)

        expected_outcome = {
            "csv": mock_file_object,
            "txt": mock_file_object
        }

        self.assertEqual(expected_outcome, test)

        mock_file_object_two = MagicMock()
        mock_file_object_two.SUPPORTED_FORMATS = ["pdf", "pickle"]

        test.add_binding(file_object=mock_file_object_two)

        expected_outcome = {
            "csv": mock_file_object,
            "txt": mock_file_object,
            "pdf": mock_file_object_two,
            "pickle": mock_file_object_two
        }

        self.assertEqual(expected_outcome, test)

        mock_file_object_three = MagicMock()
        mock_file_object_three.SUPPORTED_FORMATS = ["txt"]

        with self.assertRaises(Exception):
            test.add_binding(file_object=mock_file_object_three)
        with self.assertRaises(Exception):
            test.add_binding(file_object=mock_file_object)

    def test_get_file(self):
        test = FileMap()
        test["csv"] = MagicMock()
        mock_path = MagicMock()
        mock_path.file_type = "csv"
        out_come = test.get_file(file_path=mock_path)
        self.assertEqual(test["csv"], out_come)

        mock_path.file_type = "test"
        with self.assertRaises(Exception):
            test.get_file(file_path=mock_path)


if __name__ == "__main__":
    main()



from unittest import TestCase, main
from unittest.mock import patch

from monolith_filemanager.file.protobuf_file import ProtobufFile


class TestProtobufFile(TestCase):

    @patch("monolith_filemanager.file.protobuf_file.File.__init__")
    def test___init__(self, mock_file_init):
        ProtobufFile(path="test")
        mock_file_init.return_value = None
        mock_file_init.assert_called_once_with(path="test")

    # @patch("monolith_filemanager.file.protobuf_file.tf")
    # @patch("monolith_filemanager.file.protobuf_file.ProtobufFile.__init__")
    # def test_read_file(self, mock_init, mock_tf):
    #     mock_init.return_value = None
    #     test = ProtobufFile(path="test")
    #     test.path = "/some/path.pb"
    #     test.read()
    #
    #     mock_init.assert_called_once_with(path="test")
    #
    #     mock_tf.io.gfile.GFile.assert_called_once_with("/some/path.pb", "rb")
    #     mock_tf.compat.v1.GraphDef.assert_called_once_with()
    #
    #     mock_file = mock_tf.io.gfile.GFile.return_value.__enter__.return_value
    #     mock_graph_def = mock_tf.compat.v1.GraphDef.return_value
    #
    #     mock_graph_def.ParseFromString.assert_called_once_with(mock_file.read.return_value)
    #
    # @patch("tensorflow.io.write_graph")
    # @patch("monolith_filemanager.file.protobuf_file.ProtobufFile.__init__")
    # def test_write_file(self, mock_init, mock_write_graph):
    #     mock_init.return_value = None
    #     test = ProtobufFile(path="test")
    #     test.path = "/some/path.pb"
    #     test.write(data="data object")
    #
    #     mock_init.assert_called_once_with(path="test")
    #
    #     mock_write_graph.assert_called_once_with(
    #         graph_or_graph_def="data object",
    #         logdir="/some",
    #         name="path.pb",
    #         as_text=False
    #     )


if __name__ == "__main__":
    main()




import unittest
from unittest.mock import patch

from monolith_filemanager.file.joblib_file import JoblibFile


class TestJoblibFile(unittest.TestCase):

    @patch("monolith_filemanager.file.joblib_file.File.__init__")
    def test__init_(self, mock_file_init):
        mock_file_init.return_value = None
        JoblibFile(path='test')
        mock_file_init.assert_called_once_with(path='test')

    @patch("monolith_filemanager.file.joblib_file.joblib.load")
    @patch("monolith_filemanager.file.joblib_file.JoblibFile.__init__")
    def test_read(self, mock_init, mock_joblib_load):
        mock_init.return_value = None
        test = JoblibFile(path="test")
        test.path = "/some/path.joblib"
        test.read()

        mock_init.assert_called_once_with(path="test")
        mock_joblib_load.assert_called_once_with("/some/path.joblib")

    @patch("monolith_filemanager.file.joblib_file.joblib.dump")
    @patch("monolith_filemanager.file.joblib_file.JoblibFile.__init__")
    def test_write(self, mock_init, mock_joblib_dump):
        mock_init.return_value = None
        test = JoblibFile(path="test")
        test.path = "/some/path.joblib"
        test.write(data="string object")

        mock_init.assert_called_once_with(path="test")
        mock_joblib_dump.asssert_called_once_with(
            "string object", "/some/path.joblib"
        )


if __name__ == '__main__':
    unittest.main()



from unittest import TestCase, main
from unittest.mock import patch, MagicMock, call
from monolith_filemanager.adapters.s3_processes import S3ProcessesAdapter, S3ProcessesAdapterError


class TestS3ProcessesAdapter(TestCase):

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def setUp(self, mock_init) -> None:
        mock_init.return_value = None
        self.test_file = S3ProcessesAdapter(file_path="mock/folder/test.xlsx")
        self.test_file.path = "mock/folder/test.xlsx"
        self.test_folder = S3ProcessesAdapter(file_path="mock/folder/path")
        self.test_folder.path = "mock/folder/path"

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter._strip_path_slash")
    @patch("monolith_filemanager.adapters.s3_processes.V1Engine")
    @patch("monolith_filemanager.adapters.s3_processes.Base.__init__")
    def test___init__(self, mock_init, mock_engine, mock_strip_path_slash):
        mock_init.return_value = None
        mock_strip_path_slash.return_value = None
        test = S3ProcessesAdapter(file_path="test")

        mock_init.assert_called_once_with(file_path="test")
        mock_engine.assert_called_once_with()
        self.assertEqual(mock_engine.return_value, test._engine)
        mock_strip_path_slash.assert_called_once_with()

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_local_file_object(self, mock_init):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path="test path")
        test.file_types = MagicMock()
        test.path = MagicMock()
        out_come = test.local_file_object()

        test.file_types.get_file.assert_called_once_with(file_path=test.path)
        test.file_types.get_file.return_value.assert_called_once_with(path=test.path)
        self.assertEqual(test.file_types.get_file.return_value.return_value, out_come)

    @patch("monolith_filemanager.adapters.s3_processes.FilePath")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.local_file_object")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_read_file(self, mock_init, mock_local_file_object, mock_file_path):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path=MagicMock())
        test.path = MagicMock()
        test._cache = MagicMock()
        test._pickle_factory = MagicMock()
        test._engine = MagicMock()
        test.file_types = MagicMock()
        test.path.file_type = "pickle"

        with self.assertRaises(S3ProcessesAdapterError):
            test.read_file()

        test.path.file_type = "not pickle"
        second_cached_path = test.path.to_string.return_value
        second_out_come = test.read_file()

        test._cache.create_cache.assert_called_once_with()
        test._engine.download_data_file.assert_called_once_with(storage_path=second_cached_path,
                                                                file_path=test._cache.cache_path)
        mock_local_file_object.return_value.read.assert_called_once_with()
        mock_file_path.assert_called_once_with(test._engine.download_data_file.return_value)
        self.assertEqual(mock_file_path.return_value, test.path)
        self.assertEqual(mock_local_file_object.return_value.read.return_value, second_out_come)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_read_raw_file(self, mock_init):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path=MagicMock())
        test.path = MagicMock()
        test._engine = MagicMock()

        out_come = test.read_raw_file()
        test._engine.download_raw_data_file.assert_called_once_with(storage_path=test.path.to_string.return_value)
        self.assertEqual(out_come, test._engine.download_raw_data_file.return_value)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_custom_read_file(self, mock_init):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path="test path")
        test.path = MagicMock()
        test._cache = MagicMock()
        test._engine = MagicMock()
        test.file_types = MagicMock()
        test.path.file_type = "any"
        source_path = test.path
        custom_read_function = MagicMock(name='read function')
        data_output = MagicMock(name='data')
        custom_read_function.return_value = data_output

        out_come = test.custom_read_file(custom_read_function)

        test._cache.create_cache.assert_called_once_with()
        test._engine.download_data_file.assert_called_once_with(storage_path=source_path,
                                                                file_path=test._cache.cache_path)

        self.assertEqual(test._engine.download_data_file.return_value, test.path)
        custom_read_function.assert_called_once_with(test.path)
        self.assertEqual(out_come, data_output)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.local_file_object")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_write_file_supports_s3(self, mock_init, mock_local_file_object):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path=MagicMock())
        test._engine = MagicMock()
        test.path = MagicMock()
        mock_local_file_object.return_value.supports_s3.return_value = True
        mock_data = MagicMock()
        test.write_file(data=mock_data)
        mock_local_file_object.return_value.write.assert_called_once_with(mock_data)

    @patch("monolith_filemanager.adapters.s3_processes.FilePath")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.local_file_object")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_write_file_not_supports_s3(self, mock_init, mock_local_file_object, mock_file_path):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path=MagicMock())
        test._engine = MagicMock()
        test.path = MagicMock()
        test._cache = MagicMock()
        mock_local_file_object.return_value.supports_s3.return_value = False
        mock_local_file_object.return_value.path = test.path
        mock_data = MagicMock()
        test.write_file(data=mock_data)
        test._engine.upload_data_from_file.assert_called_once_with(file_path=mock_file_path.return_value.to_string.return_value, storage_path=test.path.to_string.return_value)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_write_raw_file(self, mock_init):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path=MagicMock())
        test._engine = MagicMock()
        test.path = MagicMock()

        mock_data = MagicMock()
        test.write_raw_file(data=mock_data)
        test._engine.upload_data.assert_called_once_with(storage_path=test.path.to_string.return_value, data=mock_data)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_delete_file(self, mock_init):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path="test path")
        test._engine = MagicMock()
        test.path = MagicMock()
        test.delete_file()

        test._engine.delete.assert_called_once_with(storage_path=test.path)

    @patch("monolith_filemanager.adapters.s3_processes.V1Engine._split_s3_path")
    def test_delete_folder(self, mock_split_path):
        mock_engine = MagicMock()
        mock_engine.delete_file.return_value = None
        self.test_folder._engine = mock_engine
        mock_split_path.return_value = ("mock-bucket", "mock/folder", "folder")

        self.test_folder.delete_folder()
        mock_engine.delete_file.assert_called_once_with(bucket_name="mock-bucket", file_name="mock/folder/")

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.increment_files")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.check_name_taken")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.ls")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_write_stream(self, mock_init, mock_ls, mock_name_taken, mock_exists, mock_increment_files):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path=MagicMock())
        test._engine = MagicMock()
        test.path = "mock/folder/file.txt"
        test._cache = MagicMock()
        mock_ls.return_value = ([], [])
        mock_stream = MagicMock()
        # test name already taken
        mock_name_taken.return_value = True
        with self.assertRaises(S3ProcessesAdapterError):
            test.write_stream(mock_stream)
        mock_stream.save.assert_called_once()

        # test already exists
        mock_stream.reset_mock()

        mock_name_taken.return_value = False
        mock_exists.return_value = True
        mock_increment_files.return_value = None
        self.assertEqual("file.txt", test.write_stream(mock_stream))
        cache_path = mock_stream.save.call_args_list[0][0][0]
        test._engine.upload_data_from_file.assert_called_once_with(storage_path=test.path, file_path=cache_path)
        mock_increment_files.assert_called_once_with()
        mock_stream.save.assert_called_once()

        # test doesn't already exist
        mock_stream.reset_mock()
        test._engine.reset_mock()
        mock_increment_files.reset_mock()
        mock_exists.return_value = False
        self.assertEqual("file.txt", test.write_stream(mock_stream))
        mock_stream.save.assert_called_once()
        cache_path = mock_stream.save.call_args_list[0][0][0]
        test._engine.upload_data_from_file.assert_called_once_with(storage_path=test.path, file_path=cache_path)
        mock_increment_files.assert_has_calls = []

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_increment_files(self, mock_init, mock_exists):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path=MagicMock())
        test.path = "mock/path/folder/file.txt"
        mock_exists.return_value = False
        test.increment_files()
        self.assertEqual("mock/path/folder/file 2.txt", test.path)

        test.path = "mock/path/folder/file.txt"
        mock_exists.side_effect = [True, False]
        test.increment_files()
        self.assertEqual("mock/path/folder/file 3.txt", test.path)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_create_directory_if_not_exists(self, mock_init):
        mock_init.return_value = None
        mock_path = MagicMock()
        test = S3ProcessesAdapter(file_path=mock_path)
        test._engine = MagicMock()
        test.path = mock_path

        test.create_directory_if_not_exists()
        test._engine.create_folder.assert_called_once_with(storage_path=mock_path)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_exists(self, mock_init):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path="test path")
        test._engine = MagicMock()
        test.path = MagicMock()

        test.exists()
        test._engine.exists.assert_called_once_with(storage_path=test.path)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_ls(self, mock_init):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path="test path")
        test._engine = MagicMock()
        test.path = MagicMock()

        test.ls()
        test._engine.ls.assert_called_once_with(storage_path=test.path.to_string.return_value)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.delete_folder")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.delete_file")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test_batch_delete(self, mock_init, mock_delete_file, mock_delete_folder):
        mock_init.return_value = None
        test = S3ProcessesAdapter(file_path="test/path")
        test.path = "test/path"
        test.path = "mock/folder/path"
        mock_paths = ["mock_folder", "mock_file.file"]
        mock_delete_file.side_effect = [None, None]
        mock_delete_folder.side_effect = [None, None]
        test.batch_delete(paths=mock_paths)
        mock_delete_folder.assert_has_calls = [call(path=test.path + mock_paths[0])]
        mock_delete_file.assert_has_calls = [call(path=test.path + mock_paths[1])]

    def test_copy_file(self):
        mock_new_path = "mock/new/path"
        mock_engine = MagicMock()
        mock_engine._split_s3_path.side_effect = [("mock-bucket", "old/file.txt", None),
                                                  ("mock-bucket", "new/file.txt", None)]
        mockObject = MagicMock()
        mockObject.copy_from.return_value = None
        mock_engine.resource.Object.return_value = mockObject

        self.test_file._engine = mock_engine
        self.test_file.copy_file(mock_new_path)
        mock_engine._split_s3_path.assert_has_calls = [call(self.test_file.path), call(mock_new_path)]
        mockObject.copy_from.assert_called_once_with(CopySource="mock-bucket/old/file.txt")

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.delete_file")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.copy_file")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.check_name_taken")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.ls")
    @patch("monolith_filemanager.adapters.s3_processes.FilePath")
    def test_rename_file(self, mock_filepath, mock_ls, mock_exists, mock_check_name_taken, mock_copy_file,
                         mock_delete_file):
        new_name = "new_name"
        mock_ext = ".xlsx"
        mock_filepath.side_effect = ["/".join(self.test_file.path.split("/")[:-1]) + f"/{new_name}" + mock_ext,
                                      "/".join(self.test_file.path.split("/")[:-1]) + "/"]
        mock_ls.return_value = ({}, [])
        mock_exists.return_value = True
        mock_check_name_taken.return_value = False

        mock_copy_file.return_value = None
        mock_delete_file.return_value = None

        self.test_file.rename_file(new_name=new_name)
        mock_ls.assert_called_once_with(path="mock/folder/")
        mock_copy_file.assert_called_once_with(new_path='mock/folder/new_name.xlsx')
        mock_delete_file.assert_called_once_with()

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.delete_folder")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.copy_folder")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.s3_processes.FilePath")
    def test_rename_folder(self, mock_filepath, mock_exists, mock_copy_folder, mock_delete_folder):
        new_name = "new_folder"
        mock_filepath.return_value = "/".join(self.test_folder.path.split("/")[:-1]) + f"/{new_name}"
        mock_exists.return_value = False
        mock_copy_folder.return_value = None
        mock_delete_folder.return_value = None
        self.test_folder.rename_folder(new_name=new_name)

        mock_copy_folder.assert_called_once_with(new_folder=f"mock/folder/{new_name}")
        mock_delete_folder.assert_called_once_with()

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.delete_file")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.copy_file")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.s3_processes.FilePath")
    def test_move_file(self, mock_filepath, mock_exists, mock_copy_file, mock_delete_file):
        mock_destination_folder = "mock/new/path"
        mock_filepath.return_value = "mock/new/path/file.txt"
        mock_exists.return_value = False

        self.test_file.move_file(destination_folder=mock_destination_folder)
        mock_copy_file.assert_called_once_with(new_path=mock_filepath.return_value)
        mock_delete_file.assert_called_once_with()

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.delete_folder")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.copy_folder")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.s3_processes.FilePath")
    def test_move_folder(self, mock_filepath, mock_exists, mock_copy_folder, mock_delete_folder):
        mock_destination_folder = "mock/new/path"
        mock_filepath.return_value = "mock/new/path/folder"
        mock_exists.return_value = False

        self.test_folder.move_folder(destination_folder=mock_destination_folder)
        mock_copy_folder.assert_called_once_with(new_folder=mock_filepath.return_value)
        mock_delete_folder.assert_called_once_with()

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.move_folder")
    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.move_file")
    @patch("monolith_filemanager.adapters.s3_processes.FilePath")
    def test_batch_move(self, mock_filepath, mock_move_file, mock_move_folder):
        mock_paths = ["test.xlsx", "test_folder"]
        mock_destination_folder = "mock/destination/folder"
        mock_filepath1 = MagicMock()
        mock_filepath1.to_string.return_value = f"{mock_destination_folder}/{mock_paths[0]}"
        mock_filepath1.get_file_type.return_value = "xlsx"
        mock_filepath2 = MagicMock()
        mock_filepath2.to_string.return_value = f"{mock_destination_folder}/{mock_paths[1]}"
        mock_filepath2.get_file_type.return_value = None
        mock_filepath.side_effect = [mock_filepath1, None, mock_filepath2, None]

        mock_move_file.return_value = None
        mock_move_folder.return_value = None

        self.test_folder.batch_move(paths=mock_paths, destination_folder=mock_destination_folder)
        mock_filepath.assert_has_calls([call(f"{mock_destination_folder}/{mock_paths[0]}"),
                                        call(f"{self.test_folder.path}/{mock_paths[0]}"),
                                        call(f"{mock_destination_folder}/{mock_paths[1]}"),
                                        call(f"{self.test_folder.path}/{mock_paths[1]}")])
        mock_move_file.assert_called_once_with(destination_folder=mock_destination_folder)
        mock_move_folder.assert_called_once_with(destination_folder=mock_destination_folder)

    @patch("monolith_filemanager.adapters.s3_processes.S3ProcessesAdapter.__init__")
    def test__strip_path_slash(self, mock_init):
        mock_init.return_value = None
        mock_path = "mock/folder/path/"
        test_folder = S3ProcessesAdapter(file_path=mock_path)
        test_folder.path = mock_path
        test_folder._strip_path_slash()
        self.assertEqual(mock_path.rstrip("/"), test_folder.path)


if __name__ == "__main__":
    main()



import numpy as np
from typing import Any, Union

from .base import File
from ..path import FilePath


class NumpyFile(File):
    """
    This is a class for managing the reading and writing of numpy objects.
    """
    SUPPORTED_FORMATS = ["npy"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the CustomDataFile class.

        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: Data from the json file
        """
        return np.load(self.path)

    def write(self, data: Any, **kwargs) -> None:
        """
        Writes data to file.

        :param data: (python object) data to be written to file
        :return: None
        """

        return np.save(self.path, data, allow_pickle=False)



from typing import Union, Any

import numpy as np

from .base import File
from .errors import VTKFileError
from ..path import FilePath


class VtkFile(File):
    """
    This is a class for managing the reading and writing of vtp files.
    """
    SUPPORTED_FORMATS = ["obj", "ply", "stl", "vtk", "vtp", "vtu"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the VtpFile class.
        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: (Any) data from file
        """
        try:
            from pyvista import read as pv_read
        except ImportError:
            raise VTKFileError(
                message="loading a vtk file relies on the vtk module. Please install this module and try again."
            )

        return expand_3d_point_arrays(pv_read(self.path))

    def write(self, data: Any, **kwargs) -> None:
        """
        Writes data to file.
        :param data: (python object) data to be written to file
        :return: None
        """
        data.save(self.path)


def expand_3d_point_arrays(mesh):
    """
    Helper function for expanding 3D point surfaces into 1D point surfaces and a magnitude vector.
    @param mesh:  (pv.PolyData) input mesh to expand
    @return: (pv.PolyData) mesh with expanded point surfaces
    """
    if all(arr.shape[-1] == 1 for arr in mesh.point_arrays.values()):
        return mesh

    new_mesh = mesh.copy()
    new_mesh.clear_point_arrays()

    for key, arr in mesh.point_arrays.items():
        if arr.shape[-1] == 3:
            new_mesh.point_arrays.update({f'{key}:X': arr[:, 0],
                                          f'{key}:Y': arr[:, 1],
                                          f'{key}:Z': arr[:, 2],
                                          f'{key}:Magnitude': np.linalg.norm(arr, axis=1),
                                          })
        else:
            new_mesh.point_arrays[key] = arr

    return new_mesh



from unittest import TestCase, main

# from machine_learning.VERSION_ONE.model.compose.scaled_regressor import ScaledInputOutputRegressor
from monolith_filemanager.adapters.version_identifier import VersionIdentifierAdapter


class MlErrorGenerator:
    pass


class TestVersionIdentifierAdapter(TestCase):

    def test___init__(self):
        test = VersionIdentifierAdapter(path="path/to/file.txt", class_object=MlErrorGenerator)
        self.assertEqual("path/to/file.txt", test._path)
        self.assertEqual(MlErrorGenerator, test._class_object)

    def test_cleaned_path(self):

        # for non-versioned object
        test = VersionIdentifierAdapter(path="path/to/file.txt", class_object=MlErrorGenerator)
        self.assertEqual("path/to/file.txt", test.cleaned_path)

        test = VersionIdentifierAdapter(path="path/to/file.pickle", class_object=MlErrorGenerator)
        self.assertEqual("path/to/file.pickle", test.cleaned_path)


if __name__ == "__main__":
    main()





class BaseFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class PandasFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class FileMapError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class MimpFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class VtpFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class Hdf5FileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class YamlFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class VtkFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class PickleFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class MatlabFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class KerasModelFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class VTKFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class ProtobufFileError(Exception):

    def __init__(self, message: str) -> None:
        super().__init__(message)


class StlObjectFileError(Exception):

    def __init(self, message: str) -> None:
        super().__init__(message)



from unittest import TestCase, main
from unittest.mock import patch
from monolith_filemanager.file.numpy_file import NumpyFile


class TestNumpyFile(TestCase):

    @patch("monolith_filemanager.file.numpy_file.File.__init__")
    def test___init__(self, mock_file_init):
        NumpyFile(path="test")
        mock_file_init.return_value = None
        mock_file_init.assert_called_once_with(path="test")

    @patch("monolith_filemanager.file.numpy_file.np")
    @patch("monolith_filemanager.file.numpy_file.NumpyFile.__init__")
    def test_read_file(self, mock_init, mock_np):
        mock_init.return_value = None
        test = NumpyFile(path="test")
        test.path = "/some/path.npy"
        test.read()

        mock_init.assert_called_once_with(path="test")
        mock_np.load.assert_called_once_with("/some/path.npy")

    @patch("monolith_filemanager.file.numpy_file.np")
    @patch("monolith_filemanager.file.numpy_file.NumpyFile.__init__")
    def test_write_file(self, mock_init, mock_np):
        mock_init.return_value = None
        test = NumpyFile(path="test")
        test.path = "/some/path.npy"
        test.write(data="string object")

        mock_init.assert_called_once_with(path="test")
        mock_np.save.assert_called_once_with("/some/path.npy", "string object", allow_pickle=False)


if __name__ == "__main__":
    main()



import os
from typing import Any, Union

from .base import File
from .errors import ProtobufFileError
from ..path import FilePath


class ProtobufFile(File):
    """
    This is a class for managing the reading and writing of tensorflow graph objects as protobuf files.
    """
    SUPPORTED_FORMATS = ["pb"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the ProtobufFile class.

        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: Data from the pb file
        """
        try:
            from tensorflow.io.gfile import GFile
            from tensorflow.compat.v1 import GraphDef
        except ImportError:
            raise ProtobufFileError(
                message="tensorflow is needed for ProtobufFile. To install run the command: 'file-install-tensorflow'"
            )
        with GFile(self.path, "rb") as file:
            graph_def = GraphDef()
            graph_def.ParseFromString(file.read())
        return graph_def

    def write(self, data: Any, **kwargs) -> None:
        """
        Writes data to file.

        :param data: (python object) data to be written to file
        :return: None
        """
        try:
            from tensorflow.io import write_graph
        except ImportError:
            raise ProtobufFileError(
                message="tensorflow is needed for ProtobufFile. To install run the command: 'file-install-tensorflow'"
            )
        directory, filename = os.path.split(self.path)
        write_graph(
            graph_or_graph_def=data,
            logdir=directory,
            name=filename,
            as_text=False
        )



from unittest import TestCase, main
from mock import patch, MagicMock
from monolith_filemanager.s3storage import V1Engine, V1EngineError


class TestV1Engine(TestCase):

    @patch("boto3.resource")
    @patch("boto3.client")
    @patch("monolith_filemanager.s3storage.FileManager.__init__")
    @patch("monolith_filemanager.s3storage.BucketManager.__init__")
    def setUp(self, bucket_init_mock, file_init_mock, mock_client, mock_resource) -> None:
        self.test = V1Engine()

    @patch("boto3.resource")
    @patch("boto3.client")
    @patch("monolith_filemanager.s3storage.FileManager.__init__")
    @patch("monolith_filemanager.s3storage.BucketManager.__init__")
    def test___init__(self, bucket_init_mock, file_init_mock, mock_client, mock_resource):
        # fire the __init__
        test = V1Engine()

        # check that the __init__ methods of the super classes are being called once without the engine self
        bucket_init_mock.assert_called_once_with(test)
        file_init_mock.assert_called_once_with(test)

        # check that the client and resource __init__ s are fired once with the 's3' argument
        mock_client.assert_called_once_with("s3")
        mock_resource.assert_called_once_with("s3")

        test_two = V1Engine()

        # test the singleton implementation
        self.assertEqual(id(test), id(test_two))

    @patch("monolith_filemanager.s3storage.FileManager.upload_serialised_data")
    def test_upload_data(self, mock_upload):
        self.test.upload_data(storage_path="s3://one/two/three", data="test")
        mock_upload.assert_called_once_with(bucket_name='one', data='test', file_name='two/three')

    @patch("monolith_filemanager.s3storage.FileManager.upload_file_from_disk")
    def test_upload_data_from_file(self, mock_upload):
        mock_file_path = MagicMock()
        self.test.upload_data_from_file(storage_path="s3://one/two/three", file_path=mock_file_path)
        mock_upload.assert_called_once_with(bucket_name='one', file_name='two/three', file_path=mock_file_path)

    @patch("monolith_filemanager.s3storage.V1Engine.increment_folder_name")
    @patch("monolith_filemanager.s3storage.FileManager.file_exists")
    @patch("monolith_filemanager.s3storage.V1Engine.ls_folder")
    @patch("monolith_filemanager.s3storage.FileManager.upload_serialised_data")
    def test_create_folder(self, mock_upload, mock_ls_folder, mock_file_exits, mock_increment_folder_name):
        # test with name clash
        mock_ls_folder.return_value = ({"file1.txt": {}, "three": {}}, ["dir1", "dir2"])
        mock_file_path = "s3://one/two/three/four"
        with self.assertRaises(V1EngineError):
            self.test.create_folder(storage_path=mock_file_path)

        # test with no name clash and no incrementing
        mock_file_exits.return_value = False
        mock_ls_folder.return_value = ({"file1.txt": {}, "file2.txt": {}}, ["dir1", "dir2"])
        mock_file_path = "s3://one/two/three/four"
        self.test.create_folder(storage_path=mock_file_path)
        mock_upload.assert_called_once_with(bucket_name="one", file_name="two/three/")
        mock_increment_folder_name.assert_has_calls = []

        # test with no name clash and with incrementing
        mock_file_exits.reset_mock()
        mock_upload.reset_mock()
        mock_increment_folder_name.return_value = "two/three 2/"
        mock_file_exits.return_value = True
        mock_ls_folder.return_value = ({"file1.txt": {}, "file2.txt": {}}, ["dir1", "dir2"])
        mock_file_path = "s3://one/two/three/four"
        self.test.create_folder(storage_path=mock_file_path)
        mock_upload.assert_called_once_with(bucket_name="one", file_name="two/three 2/")
        mock_increment_folder_name.assert_called_once_with(bucket_name="one", dirname="two/three/")

    @patch("monolith_filemanager.s3storage.file_manager.FileManager.file_exists")
    def test_increment_folder_name(self, mock_file_exists):
        mock_bucket = "mock-bucket"
        mock_dirname = "mock/folder/file"
        mock_file_exists.return_value = False

        self.test.increment_folder_name(bucket_name=mock_bucket, dirname=mock_dirname)
        self.test.client.put_object.assert_called_once_with(Bucket=mock_bucket, Key="mock/folder 2/")

        self.test.client.reset_mock()
        mock_file_exists.side_effect = [True, False]
        self.test.increment_folder_name(bucket_name=mock_bucket, dirname=mock_dirname)
        self.test.client.put_object.assert_called_once_with(Bucket=mock_bucket, Key="mock/folder 3/")

    @patch("monolith_filemanager.s3storage.FileManager.download_file_to_memory")
    def test_download_raw_data_file(self, mock_download):
        self.test.download_raw_data_file(storage_path="s3://one/two/three")
        mock_download.assert_called_once_with(bucket_name='one', file_name='two/three')

    @patch("monolith_filemanager.s3storage.FileManager.download_file_to_disk")
    def test_download_data_file(self, mock_download_file_to_disk):
        self.test.download_data_file(storage_path="s3://one/two/three", file_path="test path")
        mock_download_file_to_disk.assert_called_once_with(bucket_name="one",
                                                           file_name="two/three",
                                                           file_path='test path/three')

    @patch("monolith_filemanager.s3storage.FileManager.delete_file")
    def test_delete(self, mock_delete_file):
        self.test.delete(storage_path="s3://this/is/a/path.txt")
        mock_delete_file.assert_called_once_with(bucket_name='this', file_name='is/a/path.txt')

    @patch("monolith_filemanager.s3storage.FileManager.file_exists")
    @patch("monolith_filemanager.s3storage.FileManager.folder_exists")
    def test_file_exists(self, mock_folder_exists, mock_file_exists):
        mock_file_exists.return_value = True
        mock_folder_exists.return_value = False
        exists = self.test.exists(storage_path="s3://this/is/a/path.txt")
        mock_file_exists.assert_called_once_with(bucket_name='this', file_name='is/a/path.txt')
        self.assertTrue(exists)

    @patch("monolith_filemanager.s3storage.FileManager.file_exists")
    @patch("monolith_filemanager.s3storage.FileManager.folder_exists")
    def test_folder_exists(self, mock_folder_exists, mock_file_exists):
        mock_file_exists.return_value = False
        mock_folder_exists.return_value = True
        exists = self.test.exists(storage_path="s3://this/is/a/path.txt")
        mock_file_exists.assert_called_once_with(bucket_name='this', file_name='is/a/path.txt')
        mock_folder_exists.assert_called_once_with(bucket_name='this', file_name='is/a/path.txt')
        self.assertTrue(exists)

    @patch("monolith_filemanager.s3storage.FileManager.file_exists")
    @patch("monolith_filemanager.s3storage.FileManager.folder_exists")
    def test_neither_file_or_folder_exists(self, mock_folder_exists, mock_file_exists):
        mock_file_exists.return_value = False
        mock_folder_exists.return_value = False
        exists = self.test.exists(storage_path="s3://this/is/a/path.txt")
        mock_file_exists.assert_called_once_with(bucket_name='this', file_name='is/a/path.txt')
        mock_folder_exists.assert_called_once_with(bucket_name='this', file_name='is/a/path.txt')
        self.assertFalse(exists)

    @patch("monolith_filemanager.s3storage.FileManager.ls_folder")
    def test_ls(self, mock_ls):
        self.test.ls(storage_path="s3://this/is/a/folder")
        mock_ls.assert_called_once_with(bucket_name='this', file_name='is/a/folder')


if __name__ == "__main__":
    main()



from unittest import TestCase, main
from unittest.mock import MagicMock, patch
from monolith_filemanager.s3storage.bucket_manager import BucketManager


class TestBucketManager(TestCase):

    def test_get_buckets(self):
        test = BucketManager()

        # mock some buckets using the MagicMock object
        mocked_bucket = MagicMock()
        mocked_bucket.meta.data = {"Name": "test name"}

        mocked_bucket_two = MagicMock()
        mocked_bucket_two.meta.data = {"Name": "test name two"}

        # mock the s3 resource object and get it to return the mocked buckets
        test.resource = MagicMock()
        test.resource.buckets.all.return_value = [mocked_bucket, mocked_bucket_two]

        # fire the function
        test.get_buckets()
        # mock_check.assert_called_once_with()

        # check that the name in the meta data from the mocked buckets are in the keys
        self.assertTrue(mocked_bucket_two.meta.data['Name'] in test.buckets.keys())
        self.assertTrue(mocked_bucket.meta.data['Name'] in test.buckets.keys())

        # check that there's only 2 keys corresponding with the 2 mocked buckets
        self.assertEqual(2, len(test.buckets.keys()))

        # check that passing the name into the self.buckets dict returns the right mocked objects
        self.assertEqual(mocked_bucket_two, test.buckets[mocked_bucket_two.meta.data['Name']])
        self.assertEqual(mocked_bucket, test.buckets[mocked_bucket.meta.data['Name']])

    @patch("monolith_filemanager.s3storage.bucket_manager.BucketManager._check_bucket_cache")
    def test_get_bucket(self, mock_check):
        test = BucketManager()
        test.buckets = {"test one": 1}

        self.assertEqual(test.buckets["test one"], test.get_bucket(name="test one"))
        mock_check.assert_called_once_with()
        mock_check.reset_mock()

        with self.assertRaises(Exception):
            test.get_bucket(name="test")
        mock_check.assert_called_once_with()

    def test_get_bucket_contents(self):
        test = BucketManager()

        test.client = MagicMock()
        test.client.list_objects_v2.return_value = {"Contents": [
            {"Key": "one"}, {"Key": "two"}, {"Key": "three"}, {"Key": "four"}
        ]}
        test.get_bucket = MagicMock()
        test.get_bucket_contents(bucket_name="test")
        test.get_bucket.assert_called_once_with(name="test")

        self.assertEqual(["test"], list(test.bucket_contents.keys()))
        self.assertEqual(['keys', 'contents'], list(test.bucket_contents["test"].keys()))
        self.assertEqual(['one', 'two', 'three', 'four'], list(test.bucket_contents["test"]["keys"]))
        self.assertEqual([{'Key': 'one'}, {'Key': 'two'}, {'Key': 'three'}, {'Key': 'four'}],
                         list(test.bucket_contents["test"]["contents"]))

    def test_check_bucket_cache(self):
        test = BucketManager()
        test.get_buckets = MagicMock()

        test.buckets = [1]
        test._check_bucket_cache()

        test.buckets = []
        test._check_bucket_cache()

        test.get_buckets.assert_called_once_with()

    def test_create_bucket(self):
        test = BucketManager()
        test.resource = MagicMock()
        test.get_buckets = MagicMock()
        test._check_bucket_cache = MagicMock()

        test.create_bucket(name="test")

        test._check_bucket_cache.assert_called_once_with()
        test.resource.create_bucket.assert_called_once_with(Bucket="test")
        test.get_buckets.assert_called_once_with()

        test.buckets = {"test": 1}

        with self.assertRaises(Exception):
            test.create_bucket(name="test")

    def test_delete_bucket(self):
        test = BucketManager()
        test.resource = MagicMock()
        test.buckets = {"test": 1}
        test._check_bucket_cache = MagicMock()

        test.delete_bucket(name="test")

        test._check_bucket_cache.assert_called_once_with()
        test.resource.delete_bucket.assert_called_once_with(Bucket="test",
                                                            DeleteIfNonEmpty=False)
        self.assertEqual({}, test.buckets)

        with self.assertRaises(Exception):
            test.delete_bucket(name="test")


if __name__ == "__main__":
    main()



import subprocess
import sys


def install_flask():
    """
    Installs the flask requirement for the package.

    Returns: None
    """
    subprocess.check_call([sys.executable, '-m', 'pip', 'install', 'Flask>=1.0.0'])



from unittest import TestCase, main
from unittest.mock import patch, MagicMock
from monolith_filemanager.file.hdf5_file import Hdf5File


class TestHdf5File(TestCase):

    @patch("monolith_filemanager.file.hdf5_file.File.__init__")
    def test___init__(self, mock_file):
        Hdf5File(path="test")
        mock_file.assert_called_once_with(path="test")

    @patch("monolith_filemanager.file.hdf5_file.h5py")
    @patch("monolith_filemanager.file.hdf5_file.File.__init__")
    def test_read(self, mock_file, mock_h5py):
        mock_file.return_value = None
        test = Hdf5File(path="test")
        test.path = "test"
        out_come = test.read()
        mock_h5py.File.assert_called_once_with("test", mode='r')
        self.assertEqual(mock_h5py.File.return_value, out_come)

    @patch("monolith_filemanager.file.pickle_file.File.__init__")
    def _test_write(self, mock_file):
        mock_file.return_value = None
        # No write method to test at the moment


if __name__ == "__main__":
    main()



from typing import Union

import dill as pickle
from typing import Any

from .base import File
from ..path import FilePath


class StandardPickleFile(File):
    """
    This is a class for managing the reading and writing of pickled objects in the standard way.
    """
    SUPPORTED_FORMATS = ["sav"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the StandardPickleFile class.

        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: (Any) Data from the pickle file
        """
        with open(self.path, "rb") as file:
            loaded_data = pickle.load(file)
        return loaded_data

    def write(self, data: Any, **kwargs) -> None:
        """
        Writes data to a pickle file.

        :param data: (python object) data to be written to file
        :return: None
        """
        with open(self.path, "wb") as file:
            pickle.dump(data, file)



import unittest
from unittest.mock import patch

from monolith_filemanager.file.json_file import JSONFile


class TestJSONFile(unittest.TestCase):

    @patch("monolith_filemanager.file.json_file.File.__init__")
    def test__init_(self, mock_file_init):
        mock_file_init.return_value = None
        JSONFile(path='test')
        mock_file_init.assert_called_once_with(path='test')

    @patch("monolith_filemanager.file.json_file.json.load")
    @patch("monolith_filemanager.file.json_file.JSONFile.__init__")
    @patch("monolith_filemanager.file.json_file.open")
    def test_read(self, mock_open, mock_init, mock_json_load):
        mock_init.return_value = None
        test = JSONFile(path="test")
        test.path = "/some/path.json"
        test.read()

        mock_init.assert_called_once_with(path="test")
        mock_json_load.assert_called_once_with(mock_open.return_value.__enter__.return_value)

    @patch("monolith_filemanager.file.json_file.json.dump")
    @patch("monolith_filemanager.file.json_file.JSONFile.__init__")
    @patch("monolith_filemanager.file.json_file.open")
    def test_write(self, mock_open, mock_init, mock_json_dump):
        mock_init.return_value = None
        test = JSONFile(path="test")
        test.path = "/some/path.json"
        test.write(data="string object")

        mock_init.assert_called_once_with(path="test")
        mock_json_dump.asssert_called_once_with(
            mock_open.return_value.__enter__.return_value, "string object"
        )


if __name__ == '__main__':
    unittest.main()



from unittest import TestCase, main
from unittest.mock import patch

from monolith_filemanager.file.keras_model_file import KerasModelFile


class TestKerasModelFile(TestCase):

    @patch("monolith_filemanager.file.keras_model_file.File.__init__")
    def test___init__(self, mock_file_init):
        mock_file_init.return_value = None
        KerasModelFile(path="test")
        mock_file_init.assert_called_once_with(path="test")

    @patch("tensorflow.keras.models.load_model")
    @patch("monolith_filemanager.file.keras_model_file.KerasModelFile.__init__")
    def test_read_file(self, mock_init, mock_load_model):
        mock_init.return_value = None
        test = KerasModelFile(path="test")
        test.path = "/some/path.keras_model"
        test.read()

        mock_init.assert_called_once_with(path="test")
        mock_load_model.assert_called_once_with("/some/path.keras_model", compile=False, custom_objects={})

    @patch("tensorflow.keras.models.save_model")
    @patch("monolith_filemanager.file.keras_model_file.KerasModelFile.__init__")
    def test_write_file(self, mock_init, mock_save_model):
        mock_init.return_value = None

        test = KerasModelFile(path="test")
        test.path = "/some/path.keras_model"
        test.write(data="string object")

        mock_init.assert_called_once_with(path="test")
        mock_save_model.assert_called_once_with("string object", "/some/path.keras_model", include_optimizer=False,
                                                save_format="h5")


if __name__ == "__main__":
    main()





class S3ProcessesAdapterError(Exception):
    """
    This class is responsible for flagging errors in s3 processes.
    """
    def __init__(self, message: str) -> None:
        """
        The constructor for the S3ProcessesAdapterError class.

        :param message: (str) the error message raised when being called
        """
        super().__init__(message)


class LocalProcessesAdapterError(Exception):
    """
    This class is responsible for flagging errors in local processes.
    """
    def __init__(self, message: str) -> None:
        """
        The constructor for the LocalProcessesAdapterError class.

        :param message: (str) the error message raised when being called
        """
        super().__init__(message)



from typing import Any, Union

import numpy as np
import pandas as pd

from .base import File
from .errors import MatlabFileError
from ..file import FilePath


class MatlabFile(File):

    SUPPORTED_FORMATS = ["mat"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the MatlabFile class.

        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)
        self._file_data = {}

    def load_bytes(self, file_path: str) -> bytes:
        """
        Load the raw data from S3 or locally.

        :param file_path: (str) full file path of data to load (e.g. './demo/Battery/mydata.mat')
        :return: (bytes) raw data bytes
        """

        # Get raw data (or filename)
        if file_path not in self._file_data:
            # real_path = dataset_manager.get_dataset_file_path(file_path)
            with open(file_path) as file:
                self._file_data[file_path] = file.read()
            return self._file_data[file_path]
        else:
            return self._file_data[file_path]

    def _load_data(self, file_path: str) -> pd.DataFrame:
        """
        Load the data from a single file into a DataFrame.

        :param file_path: (str) full path of file
        :return: (DataFrame) by merging requested variables of the same shape
        """

        try:
            from scipy.io import loadmat, whosmat
        except ImportError:
            raise MatlabFileError(
                message="loading a matlab file relies on the scipy module. Please install this module and try again."
            )

        # Load full cycle data and get columns for each cycle
        bytes_obj = self.load_bytes(file_path)
        data = loadmat(bytes_obj)

        variables = list(data.keys())

        # Get all requested variables (names and shapes)
        whos = whosmat(bytes_obj)
        whos = [(name, shape, dtype) for name, shape, dtype in whos if name in variables]

        # Make sure there is only one unique shape for requested variables
        shapes = np.array([np.array(shape) for name, shape, dtype in whos])
        if len(np.unique(shapes, axis=0)) != 1:
            message = ", ".join([f"{name}: {shape}" for name, shape, dtype in whos])
            raise MatlabFileError(f'Unable to combine variables: {variables} due to different shapes: {message}')

        # Extract only requested keys
        df, n_rows = None, None
        for key in variables:

            # Get the variable flattened
            var = data[key].reshape(-1)

            # Make the DataFrame, if not created yet
            if df is None:
                n_rows = var.shape[0]
                df = pd.DataFrame(index=range(n_rows), columns=[])

            # Can only combine arrays of the same (flattened) shape
            if var.shape == (n_rows,):
                df[key] = var
            else:
                raise MatlabFileError(f'{key} flattened shape {var.shape} should be ({n_rows},)')

        return pd.DataFrame() if df is None else df

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: Data from the mat file
        """

        try:
            from scipy.io import loadmat
        except ImportError:
            raise MatlabFileError(
                message="loading a matlab file relies on the scipy module. Please install this module and try again."
            )

        return loadmat(self.path)

    def write(self, data: Any, **kwargs) -> None:
        raise MatlabFileError(message="write is not supported for matlab files")



from unittest import TestCase, main
from unittest.mock import patch, MagicMock

from monolith_filemanager.file.vtk_file import VtkFile


class TestStlFile(TestCase):

    @patch("monolith_filemanager.file.vtk_file.File.__init__")
    def test___init__(self, mock_file):
        VtkFile(path="test")
        mock_file.assert_called_once_with(path="test")

    @patch("monolith_filemanager.file.vtk_file.expand_3d_point_arrays")
    @patch("monolith_filemanager.file.vtk_file.File.__init__")
    @patch("pyvista.read")
    def test_read(self, mock_pv_read, mock_file, mock_expand_3d_point_arrays):
        mock_file.return_value = None
        test = VtkFile(path="test")
        test.path = "test"
        res = test.read()
        mock_expand_3d_point_arrays.assert_called_once_with(mock_pv_read.return_value)
        self.assertEqual(res, mock_expand_3d_point_arrays.return_value)

    @patch("monolith_filemanager.file.vtk_file.File.__init__")
    def test_write(self, mock_file):
        mock_file.return_value = None
        test = VtkFile(path="test")
        test.path = "test"
        mock_data = MagicMock()
        test.write(data=mock_data)
        mock_data.save.assert_called_once_with("test")


if __name__ == "__main__":
    main()



from typing import Any

from ..path import FilePath
from ..singleton import Singleton
from .errors import FileMapError


class FileMap(dict, metaclass=Singleton):
    """
    This is a class for managing the imports of file types.

    Attributes:
        bindings_imported bool: False if file types have not been imported, True if they have.
    """
    bindings_imported: bool

    def __init__(self) -> None:
        """
        The constructor for the FileMap class.
        """
        super().__init__()
        self.bindings_imported = False
        self.init_bindings()

    def init_bindings(self) -> None:
        """
        Imports and loads the file objects.

        :return: None
        """
        if self.bindings_imported is False:
            from monolith_filemanager.file.stl_object_file import StlObjectFile
            self.add_binding(file_object=StlObjectFile)
            from monolith_filemanager.file.hdf5_file import Hdf5File
            self.add_binding(file_object=Hdf5File)
            from monolith_filemanager.file.json_file import JSONFile
            self.add_binding(file_object=JSONFile)
            from monolith_filemanager.file.keras_model_file import KerasModelFile
            self.add_binding(file_object=KerasModelFile)
            from monolith_filemanager.file.numpy_file import NumpyFile
            self.add_binding(file_object=NumpyFile)
            from monolith_filemanager.file.pandas_file import PandasFile
            self.add_binding(file_object=PandasFile)
            from monolith_filemanager.file.standard_pickle_file import StandardPickleFile
            self.add_binding(file_object=StandardPickleFile)
            from monolith_filemanager.file.vtk_file import VtkFile
            self.add_binding(file_object=VtkFile)
            from monolith_filemanager.file.yml_file import YmlFile
            self.add_binding(file_object=YmlFile)
            from monolith_filemanager.file.joblib_file import JoblibFile
            self.add_binding(file_object=JoblibFile)
            from monolith_filemanager.file.protobuf_file import ProtobufFile
            self.add_binding(file_object=ProtobufFile)
            from monolith_filemanager.file.matlab import MatlabFile
            self.add_binding(file_object=MatlabFile)

            self.bindings_imported = True

    def add_binding(self, file_object: Any) -> None:
        """
        Adds file object to dict.

        :param file_object: (subclass of File)
        :return: None
        """
        for key in file_object.SUPPORTED_FORMATS:
            if self.get(key, None) is not None:
                raise FileMapError(
                    message="{} for the {} class has already been defined by the {} class".format(key,
                                                                                                  file_object,
                                                                                                  self.get(key)))
            self[key] = file_object

    def get_file(self, file_path: FilePath) -> Any:
        """
        Gets file object from self.

        :param file_path: (FilePath) full path to file
        :return: file object
        """
        file_class = self.get(file_path.file_type, None)
        if file_class is None:
            raise FileMapError("{} is not in the file map".format(file_path.file_type))
        return file_class



import datetime
from unittest import TestCase, main
from unittest.mock import MagicMock, patch
from monolith_filemanager.s3storage.file_manager import FileManager


class TestFileManager(TestCase):

    def test__prep_remote_name(self):
        out_come = FileManager._prep_remote_name(name="test.txt")
        self.assertEqual("test-remote.txt", out_come)

        with self.assertRaises(Exception):
            FileManager._prep_remote_name(name="test.txt.py")

        with self.assertRaises(Exception):
            FileManager._prep_remote_name(name="test")

    def test_upload_file_from_disk(self):
        test = FileManager()
        test.client = MagicMock()

        test.upload_file_from_disk(bucket_name="test-bucket", file_name="foo/bar/saved-file", file_path="local/test-file")

        test.client.upload_file.assert_called_once_with("local/test-file", "test-bucket", "foo/bar/saved-file")

    def test_upload_serialised_data(self):
        test = FileManager()
        test.client = MagicMock()

        test.upload_serialised_data(bucket_name="test-bucket", file_name="test-file",
                                    data=[1, 2, 3])

        test.client.put_object.assert_called_once_with(Body=[1, 2, 3],
                                                       Bucket="test-bucket",
                                                       Key="test-file")

        test.client.reset_mock()

        test.upload_serialised_data(bucket_name="test-bucket", file_name="test-file",
                                    data=None)

        test.client.put_object.assert_called_once_with(Bucket="test-bucket", Key="test-file")

    def test_download_file_to_disk(self):
        test = FileManager()
        test.client = MagicMock()

        test.download_file_to_disk(bucket_name="test-bucket", file_name="test.txt", file_path="test path")
        test.client.download_file.assert_called_once_with("test-bucket",
                                                          "test.txt",
                                                          "test path")

    def test_download_file_to_memory(self):
        test = FileManager()
        test.client = MagicMock()
        test.client.get_object.return_value.__getitem__.return_value.read.return_value = "test"

        out_come = test.download_file_to_memory(bucket_name="test-bucket", file_name="test.txt")

        test.client.get_object.assert_called_once_with(Bucket='test-bucket', Key='test.txt')
        self.assertEqual("test", out_come)

    def test_delete_file(self):
        test = FileManager()
        test.client = MagicMock()
        mock_resource = MagicMock()
        mock_bucket = MagicMock()
        mock_objects = MagicMock()
        mock_bucket.objects.filter.return_value = mock_objects
        mock_resource.Bucket.return_value = mock_bucket

        test.resource = mock_resource
        test.delete_file(bucket_name="test-bucket", file_name="test.txt")

        test.resource.Bucket.assert_called_once_with("test-bucket")
        mock_bucket.objects.filter.assert_called_once_with(Prefix="test.txt")
        mock_objects.delete.assert_called_once_with()

    def test_file_exists(self):
        test = FileManager()
        test.client = MagicMock()

        test.file_exists(bucket_name="test-bucket", file_name="test.txt")

        test.client.head_object.assert_called_once_with(Bucket="test-bucket", Key="test.txt")

    def test_folder_exists(self):
        test = FileManager()
        test.client = MagicMock()

        test.folder_exists(bucket_name="test-bucket", file_name="test.txt")

        test.client.list_objects.assert_called_once_with(Bucket="test-bucket", Prefix="test.txt", MaxKeys=1)

    def test_ls_folder(self):
        test = FileManager()
        test.client = MagicMock()
        paginator = test.client.get_paginator.return_value
        timestamp = datetime.datetime.utcnow()
        pages = [
            {
                'CommonPrefixes': [
                    {'Prefix': 'some/folder/foo/'},
                    {'Prefix': 'some/folder/bar/'},
                ],
                'Contents': [
                    {'Key': 'some/folder/alpha', "LastModified": timestamp, "Size": 100},
                    {'Key': 'some/folder/beta', "LastModified": timestamp, "Size": 100},
                ],
            },
            {
                'CommonPrefixes': [
                    {'Prefix': 'some/folder/baz/'},
                ],
            },
            {
                'Contents': [
                    {'Key': 'some/folder/gamma', "LastModified": timestamp, "Size": 100},
                    {'Key': 'some/folder/delta', "LastModified": timestamp, "Size": 100},
                    {'Key': 'some/folder/theta', "LastModified": timestamp, "Size": 100},
                ],
            },
        ]
        paginator.paginate.return_value = pages

        out_come = test.ls_folder(bucket_name="test-bucket", file_name="some/folder")

        test.client.get_paginator.assert_called_once_with('list_objects')
        paginator.paginate.assert_called_once_with(Bucket="test-bucket", Prefix="some/folder/", Delimiter='/')

        expected_out_come = (
            {x: {"last_modified": timestamp.strftime("%Y-%m-%d %H:%M:%S"), "size": 100} for x in
             ['alpha', 'beta', 'gamma', 'delta', 'theta']},
            ['foo', 'bar', 'baz']
        )
        self.assertEqual(expected_out_come, out_come)


if __name__ == "__main__":
    main()



from typing import Any, Union, Tuple, List, Optional
import shutil
from distutils.dir_util import copy_tree
import os
import glob
from datetime import datetime

from monolith_filemanager.adapters.base import Base
from monolith_filemanager.path import FilePath
from monolith_filemanager.errors import FileManagerError
from monolith_filemanager.file.base import File
from .errors import LocalProcessesAdapterError


class LocalFileProcessesAdapter(Base):
    """
    This is a class for managing the interface of local file commands.

    Attributes:
        python_path (str): the path of the PYTHONPATH env variable
    """
    def __init__(self, file_path: Union[str, FilePath]) -> None:
        """
        The constructor for the LocalFileProcessesAdapter class.

        :param file_path: (str) path to the file concerned
        """
        super().__init__(file_path=file_path)
        self.python_path: str = str(os.environ.get("PYTHONPATH", "").split(":")[0])

    @staticmethod
    def check_local_file(path: FilePath) -> None:
        """
        Checks to see if self.path exists locally raising errors if not.

        :param path: (FilePath) the path to be checked
        :return: None
        """
        if path.root_exists is False:
            raise FileManagerError(message="root {} does not exist".format(path.root))
        elif path.file_exists is False:
            raise FileManagerError(message="file {} does not exist".format(path.file))

    def local_file_object(self) -> File:
        """
        Gets the reading and writing protocols based on the file type from the path.

        :return: (File) containing read and write protocols for the type of file
        """
        file_object = self.file_types.get_file(file_path=self.path)
        return file_object(path=self.path)

    def export_file(self):
        """
        Exports saved file.

        :return: (flask.send_from_directory) file
        """
        try:
            from flask import send_from_directory
        except ImportError:
            raise FileManagerError(
                message=f"exporting file relies on the Flask module. "
                        f"To install the correct version run the command: `file-install-flask`."
            )
        self.check_local_file(path=self.path)
        full_path = self.python_path + "/" + self.path.root
        return send_from_directory(directory=full_path, filename=self.path.file)

    def read_file(self, **kwargs) -> Any:
        """
        Reads file.

        :return: (Any) loaded data from the file
        """
        self.check_local_file(path=self.path)
        return self.local_file_object().read(**kwargs)

    def read_raw_file(self):
        """
        Reads file.

        :return: (bytes) loaded data from the file
        """
        self.check_local_file(path=self.path)
        with open(self.path, "rb") as f:
            return f.read()

    def custom_read_file(self, custom_read_function: Any) -> Any:
        """
        Takes a custom reading function and runs it in order to read a file based on the self.path.

        :param custom_read_function: (Any) the function that is going to read the file
        :return: (Any) the return value from the custom_read_function
        """
        self.check_local_file(path=self.path)
        return custom_read_function(self.path)

    def write_file(self, data, **kwargs):
        """
        Writes data to file.

        :param data: data to be written to file
        :return: None
        """
        self._create_directory_if_not_exists()
        self.local_file_object().write(data=data, **kwargs)

    def write_raw_file(self, data):
        """
        Writes data to file.

        :param data: data to be written to file
        :return: None
        """
        self._create_directory_if_not_exists()
        with open(self.path, "wb") as f:
            f.write(data)

    def delete_file(self, path: Optional[FilePath] = None) -> None:
        """
        Deletes local file.

        :param path: (Optional[FilePath])
        :return: None
        """
        if not path:
            path = self.path
        self.check_local_file(path=path)
        if os.path.isdir(path):
            shutil.rmtree(path)
        elif os.path.isfile(path):
            os.remove(path)
        else:
            raise FileManagerError("Could not delete file/folder at path: " + path)

    def delete_folder(self, path: Optional[FilePath] = None) -> None:
        """
        Calls delete file method which also deletes directories.

        :param path: (Optional[FilePath])
        :return: None
        """
        if not path:
            path = self.path
        self.delete_file(path=path)

    def write_stream(self, stream) -> str:
        """
        Write stream of bytes. First checks whether file name shares name of existing folder in same directory level.
        If file with same name exists already, file name is incremented before writing.
        :param stream: stream of data
        :return: (str) Name file saved as
        :raises: (LocalProcessesAdapterError) if new file name taken by existing folder
        """
        if os.path.isdir(self.path):
            raise LocalProcessesAdapterError("New file name already taken by folder in this folder")
        elif os.path.isfile(path=self.path):
            self.increment_files()

        self._create_directory_if_not_exists()
        stream.save(self.path)
        file_name = self.path.split("/")[-1]
        return file_name

    def increment_files(self) -> None:
        """
        Increments file name integer suffix based on how many files with same filename prefix exist.
        Fills in gaps in contiguous numeric sequence first. Assigns final name to self.path.
        :return: None
        """
        count = 2
        prefix = "/".join(self.path.split("/")[:-1])
        ext = os.path.splitext(self.path)[-1]
        file = ".".join(self.path.split("/")[-1].split(".")[:-1])
        new_file = prefix + "/" + file + f" {count}{ext}"
        while os.path.exists(new_file):
            count += 1
            new_file = prefix + "/" + file + f" {count}{ext}"
        self.path = new_file

    def _create_directory_if_not_exists(self) -> None:
        """
        Creates a directory if it does not exist. Used privately in adapter.
        :return: None
        """
        dirname = os.path.dirname(self.path)
        if not os.path.exists(dirname):
            os.makedirs(dirname)

    def create_directory_if_not_exists(self) -> str:
        """
        Public method to explicitly create a directory (if it does not exist) when passed a filepath.
        :return: (str) new folder name
        :raises: (LocalProcessesAdapterError) If new folder name already taken by file in directory
        """
        dirname = FilePath(os.path.dirname(self.path))
        folder = dirname.split("/")[-1]

        if os.path.isfile(path=dirname):
            raise LocalProcessesAdapterError("New folder name already taken by file in this folder")
        elif os.path.isdir(dirname):
            folder = self.increment_folders(dirname=dirname)
        else:
            os.makedirs(dirname)
        return folder

    @staticmethod
    def increment_folders(dirname: FilePath) -> str:
        """
        Increment folder name suffix if name prefix already exists within parent directory.
        :param dirname: (FilePath) folder/directory name to increment
        :return: (str) new folder name
        """
        count = 2
        prefix = "/".join(dirname.split("/")[:-1])
        folder = dirname.split("/")[-1]
        new_folder = prefix + "/" + folder + f" {count}"
        while os.path.exists(new_folder):
            count += 1
            new_folder = prefix + "/" + folder + f" {count}"
        os.makedirs(new_folder)
        return new_folder.split("/")[-1]

    def exists(self, path: Optional[FilePath] = None) -> bool:
        """
        Checks to see if the self.path exists

        :param path: (Optional[FilePath]) if not passed, defaults to self.path
        :return: (bool) True if exists, False if not
        """
        if not path:
            path = self.path
        return os.path.exists(path)

    def ls(self) -> Tuple[dict, List[str]]:
        """
        Lists the directories and files in a path.

        :return: (Tuple[dict, List[str]]) [file_dict, directories]
        """
        if "~" in self.path:
            self.path = os.path.expanduser(self.path)
        root, dirs, files = next(os.walk(self.path))
        file_dict = {}
        for file in files:
            last_mod, size = self._get_file_info(os.path.join(self.path, file))
            file_dict[file] = {"last_modified": last_mod, "size": size}
        return file_dict, dirs

    @staticmethod
    def _get_file_info(file: str) -> Tuple[str, int]:
        """
        Returns last modified date and file size in bytes.
        :param file: (str) local file
        :return: (Tuple[str, int]) last modified date and size in bytes
        """
        last_mod = datetime.utcfromtimestamp(os.path.getmtime(file)).strftime("%Y-%m-%d %H:%M:%S")
        size = os.path.getsize(file)
        return last_mod, size

    def search(self, file_pattern: str) -> List[str]:
        """
        Gets paths for all files matching a glob pattern.

        :param file_pattern: (str) glob pattern of the file being searched for
        :return: list of strings which are paths to the found files
        """
        buffer: List[str] = []
        matches: List[str] = glob.glob(self.path + "/*", recursive=True)
        for path in matches:
            if file_pattern in path:
                buffer.append(path)
        return buffer

    def copy_folder(self, new_folder: str) -> None:
        """
        Copies a folder and it's contents from self.path to a path defined in the function.

        :param new_folder: (str) the folder that the self.path folder is being copied to
        :return: None
        """
        copy_tree(self.path, new_folder)

    def rename_file(self, new_name: str) -> None:
        """
        Checks file exists and new name is not already taken. Renames file according to provided arg.

        :param new_name: (str) new file name without extension
        :return: None
        :raises: (LocalProcessesAdapterError) if file doesn't exist or name already taken
        """
        ext = os.path.splitext(self.path)[-1]
        new_path = FilePath("/".join(self.path.split("/")[:-1]) + f"/{new_name}" + ext)
        if not self.exists():
            raise LocalProcessesAdapterError("File does not exist to rename")
        if self.exists(path=new_path):
            raise LocalProcessesAdapterError("New file name already taken by a file or directory in this folder")

        os.rename(self.path, new_path)

    def rename_folder(self, new_name: str) -> None:
        """
        Checks folder exists and new name is not already taken. Renames folder according to provided arg.

        :param new_name: (str) new folder name
        :return: None
        :raises: (LocalProcessesAdapterError) if folder does not exist or new name is taken
        """
        new_path = FilePath("/".join(self.path.split("/")[:-1]) + f"/{new_name}")
        if not self.exists():
            raise LocalProcessesAdapterError("Folder does not exist to rename")
        if self.exists(path=new_path):
            raise LocalProcessesAdapterError("New file name already taken by a file or directory in this folder")

        os.rename(self.path, new_path)

    def _move(self, new_path: FilePath) -> None:
        """
        Private method shared by both move_file and move_folder methods.
        :param new_path: (FilePath) new path to move self.path to
        :return: None
        :raises: (LocalProcessesAdapterError) if new path name already taken
        """
        if self.exists(path=new_path):
            raise LocalProcessesAdapterError("File/Folder name already taken by a file or directory in this folder")
        os.rename(self.path, new_path)

    def move_file(self, destination_folder: str) -> None:
        """
        Public move file method which formats new_path and calls _move.

        :param destination_folder: (str) folder self.path file to be moved to
        :return: None
        """
        file = self.path.split("/")[-1]
        new_path = FilePath(f"{destination_folder}/{file}")
        self._move(new_path=new_path)

    def move_folder(self, destination_folder: str) -> None:
        """
        Public move folder method which formats new_path and calls _move.

        :param destination_folder: (str) folder self.path folder to be moved to
        :return: None
        """
        folder = self.path.split("/")[-1]
        new_path = FilePath(f"{destination_folder}/{folder}")
        self._move(new_path=new_path)

    def batch_move(self, paths: List[str], destination_folder: str) -> None:
        """
        Iterates through list of paths within the self.path directory, ascertains if file or folder, then calls
        appropriate move method.

        :param paths: (List[str]) file and folder paths to be moved
        :param destination_folder: (str) folder path to be moved to
        :return: None
        """
        origin_folder = self.path
        for path in paths:
            existing_path = FilePath(f"{self.path}/{path}")
            if os.path.isfile(existing_path):
                self.path = f"{self.path}/{path}"
                self.move_file(destination_folder=destination_folder)
            else:
                self.path = f"{self.path}/{path}"
                self.move_folder(destination_folder=destination_folder)
            self.path = origin_folder



from unittest import TestCase, main
from unittest.mock import patch
from monolith_filemanager.file.base import File


class TestBase(TestCase):

    @patch.multiple(File, __abstractmethods__=set())
    @patch("monolith_filemanager.file.base.FilePath.__init__")
    def test___init__(self, mock_init):
        mock_init.return_value = None

        with self.assertRaises(Exception):
            File(path=1)

        input_path = "test"

        test = File(path=input_path)
        self.assertEqual(input_path, test.path)

        with patch("monolith_filemanager.file.base.isinstance") as mock_instance:
            mock_instance.return_value = True
            test = File(path=input_path)
            self.assertEqual(input_path, test.path)

        mock_init.assert_called_once_with(input_path)


if __name__ == "__main__":
    main()



from typing import Dict, List, Union

from ..path import FilePath


class PathMap(dict):
    """
    This class is responsible for mapping the root of a path to another path.
    """
    def __init__(self, file_map: Dict) -> None:
        """
        The constructor for the PathMap class.

        :param file_map: (Dict) config dictionary to act as a map
        """
        super().__init__(file_map)

    def map_path(self, path: Union[FilePath, str]) -> FilePath:
        """
        Maps the path to the path defined in the config.

        :param path: (Union[FilePath, str]) path to be mapped
        :return: (FilePath) path that has been rerouted
        """
        buffer: List[str] = path.split("/")

        if self.get(buffer[0]) is not None:
            buffer[0] = self[buffer[0]]

        return FilePath("/".join(buffer))



from unittest import TestCase, main
from unittest.mock import patch, MagicMock, call

from monolith_filemanager.adapters.local_file_processes import LocalFileProcessesAdapter, LocalProcessesAdapterError


class TestLocalFileProcessesAdapterWithoutInit(TestCase):

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def setUp(self, mock_init) -> None:
        # todo - flesh this out later to apply to more test cases
        mock_init.return_value = None
        self.test_folder = LocalFileProcessesAdapter(file_path="mock/folder/path", caching=MagicMock())
        self.test_folder.path = "mock/folder/path"
        self.test_file = LocalFileProcessesAdapter(file_path="mock/folder/test.xlsx", caching=MagicMock())
        self.test_file.path = "mock/folder/test.xlsx"

    @patch("monolith_filemanager.adapters.local_file_processes.os.environ", {"PYTHONPATH": "mock/python/path"})
    def test___init__local(self):
        mock_path = "mock/folder/path"
        obj = LocalFileProcessesAdapter(file_path=mock_path)
        self.assertIsInstance(obj, LocalFileProcessesAdapter)
        self.assertEqual(obj.path, mock_path)
        self.assertEqual(obj._s3, False)
        self.assertEqual(obj.python_path, "mock/python/path")

    @patch("monolith_filemanager.adapters.local_file_processes.os.environ", {})
    def test___init__local_missing_pythonpath(self):
        mock_path = "mock/folder/path"
        obj = LocalFileProcessesAdapter(file_path=mock_path)
        self.assertIsInstance(obj, LocalFileProcessesAdapter)
        self.assertEqual(obj.path, mock_path)
        self.assertEqual(obj._s3, False)
        self.assertEqual(obj.python_path, "")

    def test_check_local_file(self):
        mock_path = MagicMock()

        mock_path.root_exists = True
        mock_path.file_exists = True
        LocalFileProcessesAdapter.check_local_file(path=mock_path)

        mock_path.root_exists = False
        mock_path.file_exists = True
        with self.assertRaises(Exception):
            LocalFileProcessesAdapter.check_local_file(path=mock_path)

        mock_path.root_exists = True
        mock_path.file_exists = False
        with self.assertRaises(Exception):
            LocalFileProcessesAdapter.check_local_file(path=mock_path)

        mock_path.root_exists = False
        mock_path.file_exists = False
        with self.assertRaises(Exception):
            LocalFileProcessesAdapter.check_local_file(path=mock_path)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_local_file_object(self, mock_init):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.file_types = MagicMock()
        test.path = MagicMock()
        out_come = test.local_file_object()

        test.file_types.get_file.assert_called_once_with(file_path=test.path)
        test.file_types.get_file.return_value.assert_called_once_with(path=test.path)
        self.assertEqual(test.file_types.get_file.return_value.return_value, out_come)

    @patch("flask.send_from_directory")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.check_local_file")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_export_file(self, mock_init, mock_check_local_file, mock_send_from_directory):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()
        test.path.root = "test.csv"
        test.python_path = "python/path"

        out_come = test.export_file()

        mock_check_local_file.assert_called_once_with(path=test.path)
        mock_send_from_directory.assert_called_once_with(directory="python/path/test.csv", filename=test.path.file)
        self.assertEqual(out_come, mock_send_from_directory.return_value)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.local_file_object")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.check_local_file")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_read_file(self, mock_init, mock_check, mock_local_file_object):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()
        test.path.file_type = "not pickle"

        out_come = test.read_file()
        mock_check.assert_called_once_with(path=test.path)
        self.assertEqual(mock_local_file_object.return_value.read.return_value, out_come)

    @patch("monolith_filemanager.adapters.local_file_processes.open")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.check_local_file")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_read_raw_file(self, mock_init, mock_check, mock_open):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()

        out_come = test.read_raw_file()
        mock_check.assert_called_once_with(path=test.path)
        mock_open.assert_called_once_with(test.path, "rb")
        mock_file_returned = mock_open.return_value.__enter__.return_value
        self.assertEqual(mock_file_returned.read.return_value, out_come)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.check_local_file")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_custom_read_file(self, mock_init, mock_check):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()
        test.path.file_type = "not pickle"
        custom_read_function = MagicMock(name='read function')
        data_output = MagicMock(name='data')
        custom_read_function.return_value = data_output

        out_come = test.custom_read_file(custom_read_function)
        mock_check.assert_called_once_with(path=test.path)
        custom_read_function.assert_called_once_with(test.path)
        self.assertEqual(out_come, data_output)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter._create_directory_if_not_exists")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.local_file_object")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_write_file(self, mock_init, mock_local_file_object, mock_create_dir):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()
        mock_data = MagicMock()

        test.write_file(data=mock_data)
        mock_create_dir.assert_called_once()
        mock_local_file_object.return_value.write.assert_called_once_with(data=mock_data)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter._create_directory_if_not_exists")
    @patch("monolith_filemanager.adapters.local_file_processes.open")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_write_raw_file(self, mock_init, mock_open, mock_create_dir):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()
        mock_data = MagicMock()

        test.write_raw_file(data=mock_data)
        mock_create_dir.assert_called_once()
        mock_open.assert_called_once_with(test.path, "wb")
        mock_file_returned = mock_open.return_value.__enter__.return_value
        mock_file_returned.write.assert_called_once_with(mock_data)

    @patch("monolith_filemanager.adapters.local_file_processes.os")
    @patch("monolith_filemanager.adapters.local_file_processes.shutil")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.check_local_file")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_delete_file_directory(self, mock_init, mock_check, mock_shutil, mock_os):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()
        mock_os.path = MagicMock()
        mock_os.path.isdir.return_value = True
        mock_os.path.isfile.return_value = False

        test.delete_file()
        mock_check.assert_called_once_with(path=test.path)
        mock_shutil.rmtree.assert_called_once_with(test.path)

    @patch("monolith_filemanager.adapters.local_file_processes.os")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.check_local_file")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_delete_file_single(self, mock_init, mock_check, mock_os):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()
        mock_os.path = MagicMock()
        mock_os.path.isdir.return_value = False
        mock_os.path.isfile.return_value = True

        test.delete_file()
        mock_check.assert_called_once_with(path=test.path)
        mock_os.remove.assert_called_once_with(test.path)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.delete_file")
    def test_delete_folder(self, mock_delete_file):
        mock_delete_file.return_value = None
        self.test_folder.delete_folder()
        mock_delete_file.assert_called_once_with(path=self.test_folder.path)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter._create_directory_if_not_exists")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.increment_files")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.isfile")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.isdir")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_write_stream(self, mock_init, mock_os_path_isdir, mock_os_path_isfile, mock_increment_files,
                          mock_create_dir):
        # test if name taken
        mock_init.return_value = None
        mock_os_path_isdir.return_value = True
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = "mock/path/folder/file.txt"
        mock_stream = MagicMock()

        with self.assertRaises(LocalProcessesAdapterError):
            test.write_stream(stream=mock_stream)

        # test file exists so increment
        mock_os_path_isdir.return_value = False
        mock_os_path_isfile.return_value = True
        self.assertEqual("file.txt", test.write_stream(stream=mock_stream))
        mock_increment_files.assert_called_once_with()
        mock_create_dir.assert_called_once()
        mock_stream.save.assert_called_once_with(test.path)

        # test file does not exist, no increment
        mock_os_path_isdir.return_value = False
        mock_os_path_isfile.return_value = False
        mock_create_dir.reset_mock()
        mock_stream.reset_mock()
        self.assertEqual("file.txt", test.write_stream(stream=mock_stream))
        mock_create_dir.assert_called_once()
        mock_stream.save.assert_called_once_with(test.path)

    @patch("monolith_filemanager.adapters.local_file_processes.os.path.exists")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_increment_files(self, mock_init, mock_exists):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = "mock/path/folder/file.txt"
        mock_exists.return_value = False
        test.increment_files()
        self.assertEqual("mock/path/folder/file 2.txt", test.path)

        test.path = "mock/path/folder/file.txt"
        mock_exists.side_effect = [True, False]
        test.increment_files()
        self.assertEqual("mock/path/folder/file 3.txt", test.path)

    @patch("monolith_filemanager.adapters.local_file_processes.os.makedirs")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.increment_folders")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.isdir")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.isfile")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.dirname")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_create_directory_if_not_exists(self, mock_init, mock_os_path_dirname, mock_os_path_is_file,
                                            mock_os_path_is_dir, mock_increment_folders, mock_os_makedirs):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = "mock/pardir/folder/file"
        # test if name taken
        mock_os_path_dirname.side_effect = ["mock/pardir/folder", "mock/pardir"]
        mock_os_path_is_file.return_value = True
        with self.assertRaises(LocalProcessesAdapterError):
            test.create_directory_if_not_exists()

        # test if doesn't exist
        mock_os_path_is_file.return_value = False
        mock_os_path_is_dir.return_value = False
        mock_os_path_dirname.side_effect = ["mock/pardir/folder", "mock/pardir"]
        self.assertEqual("folder", test.create_directory_if_not_exists())
        mock_os_makedirs.assert_called_once_with("mock/pardir/folder")

        # test if already exists
        mock_os_path_is_file.return_value = False
        mock_os_path_is_dir.return_value = True
        mock_increment_folders.return_value = "folder 2"
        mock_os_path_dirname.side_effect = ["mock/pardir/folder", "mock/pardir"]
        self.assertEqual("folder 2", test.create_directory_if_not_exists())
        mock_increment_folders.assert_called_once_with(dirname="mock/pardir/folder")

    @patch("monolith_filemanager.adapters.local_file_processes.os.makedirs")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.exists")
    def test_increment_folders(self, mock_os_path_exists, mock_os_makedirs):
        mock_dirname = "mock/pardir/folder"
        mock_os_path_exists.return_value = False
        LocalFileProcessesAdapter.increment_folders(dirname=mock_dirname)
        mock_os_makedirs.assert_called_once_with("mock/pardir/folder 2")

        mock_os_makedirs.reset_mock()
        mock_os_path_exists.side_effect = [True, False]
        LocalFileProcessesAdapter.increment_folders(dirname=mock_dirname)
        mock_os_makedirs.assert_called_once_with("mock/pardir/folder 3")

    @patch("monolith_filemanager.adapters.local_file_processes.os.path")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_exists(self, mock_init, mock_os_path):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = MagicMock()

        test.exists()
        mock_os_path.exists.assert_called_once_with(test.path)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter._get_file_info")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.join")
    @patch("monolith_filemanager.adapters.local_file_processes.os.walk")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_ls(self, mock_init, mock_os_walk, mock_os_path_join, mock_get_file_info):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test path", caching=MagicMock())
        test.path = "mock/path"
        mock_os_walk.return_value = iter([["mock/root", ["dir1", "dir2"], ["file1.txt"]]])
        mock_os_path_join.return_value = "mock/path/file1.txt"
        mock_get_file_info.return_value = "last_mod_date", 100

        self.assertEqual(({"file1.txt": {"last_modified": "last_mod_date", "size": 100}}, ["dir1", "dir2"]), test.ls())

    @patch("monolith_filemanager.adapters.local_file_processes.os.path.getsize")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.getmtime")
    def test__get_file_info(self, mock_path_getmtime, mock_path_get_size):
        mock_file = MagicMock()
        mock_path_getmtime.return_value = 0
        mock_path_get_size.return_value = 100
        self.assertEqual(('1970-01-01 00:00:00', 100), LocalFileProcessesAdapter._get_file_info(mock_file))

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.delete_file")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.__init__")
    def test_batch_delete(self, mock_init, mock_delete_file):
        mock_init.return_value = None
        test = LocalFileProcessesAdapter(file_path="test/path", caching=MagicMock())
        test.path = "test/path"
        test.path = "mock/folder/path"
        mock_paths = ["mock_folder", "mock_file"]
        mock_delete_file.side_effect = [None, None]
        test.batch_delete(paths=mock_paths)
        mock_delete_file.assert_has_calls = [call(path=test.path + mock_paths[0]),
                                             call(path=test.path + mock_paths[1])]

    @patch("monolith_filemanager.adapters.local_file_processes.os.rename")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.splitext")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.dirname")
    def test_rename_file(self, mock_os_dirname, mock_os_splitext, mock_exists, mock_os_rename):
        new_name = "new_file"
        mock_os_dirname.return_value = "/".join(self.test_file.path.split("/")[:-1])
        mock_os_splitext.return_value = ["mock/folder/test", ".xlsx"]
        mock_exists.side_effect = [True, False]

        new_path = "/".join(self.test_file.path.split("/")[:-1]) + f"/{new_name}" + mock_os_splitext.return_value[-1]
        self.test_file.rename_file(new_name=new_name)
        mock_os_rename.assert_called_once_with(self.test_file.path, new_path)

    @patch("monolith_filemanager.adapters.local_file_processes.os.rename")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.exists")
    @patch("monolith_filemanager.adapters.local_file_processes.os.walk")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.dirname")
    def test_rename_folder(self, mock_os_dirname, mock_os_walk, mock_exists, mock_rename):
        new_name = "new_folder"
        mock_os_dirname.return_value = "/".join(self.test_folder.path.split("/")[:-1])
        mock_os_walk.return_value = iter([["mock/root", ["dir1", "dir2"], ["file1.txt"]]])
        mock_exists.side_effect = [True, False]

        new_path = "/".join(self.test_folder.path.split("/")[:-1]) + f"/{new_name}"
        self.test_folder.rename_file(new_name=new_name)
        mock_rename.assert_called_once_with(self.test_folder.path, new_path)

    @patch("monolith_filemanager.adapters.local_file_processes.os.rename")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.exists")
    def test__move(self, mock_exists, mock_os_rename):
        mock_exists.return_value = False
        mock_os_rename.return_value = None
        mock_new_path = MagicMock()
        self.test_folder._move(mock_new_path)
        mock_os_rename.assert_called_once_with(self.test_folder.path, mock_new_path)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter._move")
    @patch("monolith_filemanager.adapters.local_file_processes.FilePath")
    def test_move_file(self, mock_file_path, mock_move):
        mock_file_path.return_value = MagicMock()
        mock_move.return_value = None
        mock_destination_folder = "mock/destination/folder"
        self.test_file.move_file(f"{mock_destination_folder}")
        mock_file_path.assert_called_once_with(f"{mock_destination_folder}/{self.test_file.path.split('/')[-1]}")
        mock_move.assert_called_once_with(new_path=mock_file_path.return_value)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter._move")
    @patch("monolith_filemanager.adapters.local_file_processes.FilePath")
    def test_move_folder(self, mock_file_path, mock_move):
        mock_file_path.return_value = MagicMock()
        mock_move.return_value = None
        mock_destination_folder = "mock/destination/folder"
        self.test_folder.move_file(f"{mock_destination_folder}")
        mock_file_path.assert_called_once_with(f"{mock_destination_folder}/{self.test_folder.path.split('/')[-1]}")
        mock_move.assert_called_once_with(new_path=mock_file_path.return_value)

    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.move_folder")
    @patch("monolith_filemanager.adapters.local_file_processes.LocalFileProcessesAdapter.move_file")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.isdir")
    @patch("monolith_filemanager.adapters.local_file_processes.os.path.isfile")
    @patch("monolith_filemanager.adapters.local_file_processes.FilePath")
    def test_batch_move(self, mock_filepath, mock_os_isfile, mock_os_isdir, mock_move_file, mock_move_folder):
        mock_paths = ["test.xlsx", "test_folder"]
        mock_destination_folder = "mock/destination/folder"
        mock_filepath.side_effect = [MagicMock(), MagicMock()]
        mock_os_isfile.side_effect = [True, False]
        mock_os_isdir.side_effect = [False, True]
        mock_move_file.return_value = None
        mock_move_folder.return_value = None

        self.test_folder.batch_move(paths=mock_paths, destination_folder=mock_destination_folder)
        mock_filepath.assert_has_calls([call(f"{self.test_folder.path}/{mock_paths[0]}"),
                                        call(f"{self.test_folder.path}/{mock_paths[1]}")])
        mock_move_file.assert_called_once_with(destination_folder=mock_destination_folder)
        mock_move_folder.assert_called_once_with(destination_folder=mock_destination_folder)


if __name__ == "__main__":
    main()



import os


class FilePath(str):
    """
    This is a class for managing the path to a file.

    Attributes:
        file (str): name of file at the end of path
        file_type (str): file extension
        root (str): string leading up to the file
        s3 (bool): True if pointing to s3
    """
    def __init__(self, path: str) -> None:
        """
        The constructor for the FilePath class.

        :param path: (str) path to the file
        """
        super().__init__()
        self.file: str = path.split("/")[-1]
        self.file_type: str = self.get_file_type(file_string=self.file)
        self.root: str = "/".join(path.split("/")[0:-1]) + "/"
        self.s3: bool = self.check_if_s3(path_string=path)

    @staticmethod
    def check_if_s3(path_string: str) -> bool:
        """
        Checks to see if the path is pointing to an s3 bucket.

        :param path_string: (str) path to the file
        :return: True if s3, False if not
        """
        return path_string.startswith("s3://")

    @staticmethod
    def get_file_type(file_string: str) -> str:
        """
        Gets file extension.

        :param file_string: (str) file to be checked
        :return: (str) file extension
        """
        return file_string.split(".")[-1] if "." in file_string else None

    @property
    def root_exists(self) -> bool:
        """
        Checks to see if the root exists.

        :return: True if exists, False if not
        """
        return os.path.isdir(self.root)

    @property
    def file_exists(self) -> bool:
        """
        Checks to see if the file exists.

        :return: True if exists, False if not
        """
        return os.path.exists(self)

    def to_string(self) -> str:
        """
        Packages self to string.

        :return: (str) packaged self
        """
        return str(self)



import json
from typing import Any, Union

from monolith_filemanager.file.base import File
from ..path import FilePath


class JSONFile(File):
    """
    This is a class for managing the reading and writing of json objects.
    """
    SUPPORTED_FORMATS = ["json"]

    def __init__(self, path: Union[str, FilePath]) -> None:
        """
        The constructor for the JSONFile class.
        :param path: (str/FilePath) path to the file
        """
        super().__init__(path=path)

    def read(self, **kwargs) -> Any:
        """
        Gets data from file defined by the file path.

        :return: Data from the json file
        """
        with open(self.path) as data_file:
            data = json.load(data_file)
        return data

    def write(self, data: Any, **kwargs) -> None:
        """
        Writes data to file.

        :param data: (python dict) data to be written to file
        """
        with open(self.path, 'w') as data_file:
            json.dump(data, data_file)



from typing import Any


class VersionIdentifierAdapter:
    """
    This is a class for changing the path file extension to ".sav" if the object is version controlled.

    Properties:
        cleaned_path (str): path that's modified if it's a ".pickle" file and it's version controlled
                            else => unaltered path.
    """
    def __init__(self, path: str, class_object: Any) -> None:
        """
        The constructor for the VersionIdentifierAdapter.

        :param path: (str) path to the file to be saved
        :param class_object: (Any) object to be saved
        """
        self._path: str = path
        self._class_object: Any = class_object

    @property
    def cleaned_path(self) -> str:
        """
        Alters the self._path with a ".sav" if ".pickle" is present and the object is versioned.

        :return: (str) cleaned path
        """
        if ".pickle" in self._path and (hasattr(self._class_object, "VERSION") or "VERSION" in self._class_object.__module__):
            return self._path.replace(".pickle", ".sav")
        else:
            return self._path
